      program gme
c 
c     ******************************************************************
c
c     Calculates photonic structure of two-dimensional photonic crystals 
c     in an asymmetric waveguide made of three layers 1,2,3.
c     Written by L.C. Andreani, Department of Physics "A. Volta",
c     University of Pavia, Italy. e-mail andreani@fisicavolta.unipv.it
c
c     Version 15 June 2006
c
c     ******************************************************************
c
      implicit none
      integer jlayer,jcomp,nxgrid,nygrid,nzgrid,jxgrid,jygrid,jzgrid
      integer jfields,jkeigenv,jstate
      integer jlattice,jbasis,n1max,n1,n2,maxg,maxpw,npw,maxham,nrec
      integer nk,nktot,jk,maxk,j1,j2,nham,ifail,jband,jeigenv,nbands,job
      integer jx,jy,ncount,nalpha,jparxy,jparkz,jal1,jal2
      integer maxen,maxgrid,maxbands,maxalpha,jham1,jham2,jom,nom,jfun,j
      integer nalpha1,jmode1,jmode2,niter,neffet,jfreephotons,n2max,nkh
      integer nham_store,jshift,nvar,jvar,jloop,jlossmin,jlossmax
      real*8 pi,g,upper,fill,delta_kronecker,dummy,rcond,phi
      real*8 alength1,alength2,alength3,alength4,alattice
      real*8 bl1,bl2,supercellperiod,factor
      real*8 ft11,ft12,ft13,ft99,ft98,ft22,ft40,ft41
      real*8 a1(2),a2(2),aret,ucarea,b1(2),b2(2),bzarea,aacos,aasin
      real*8 d,d_store,om,om_clad,om_core,cosint,sinint,scal
      real*8 om_clad1,om_clad3,chi3,q,znorm,spher0,qp,chi1p,chi3p
      real*8 elnorm,eps_realspace
      real*8 agx1,agy1,ag1,q1,chi1,omega1,agx2,agy2,ag2,q2,chi2,omega2
      real*8 pol1(2),qol1(2),pol2(2),qol2(2),funte,funtm,pivot,rivale
      real*8 fwaveguide,flag,error,diff,axmult,axsum
      real*8 freq2,freq,akmod,alight,freq2im,freqim,dos,akz1,akz2,akz3
      real*8 varmin,varmax,delvar,var,diff1,diff2
      real*8 delkx,delky,delk,epsilon
      real*8 xmin,xmax,delx,ymin,ymax,dely,zmin,zmax,delz,xx,yy,zz
      external fwaveguide
      complex*16 i,elemte,elemtm,couplte,coupltm,t11,t12,t21,t22
      parameter (aret=1d0,maxg=2e5,maxk=1000)
      parameter (maxen=1000,maxgrid=2500,maxbands=100,upper=1d-12)
      parameter (maxalpha=8,maxpw=1100,maxham=2200,i=(0d0,1d0))
      real*8 omlow(maxalpha),omupp(maxalpha),emod2,bmod2
      real*8 om_store(maxpw,maxalpha),q_store(maxpw,maxalpha)
      real*8 chi1_store(maxpw,maxalpha),chi3_store(maxpw,maxalpha)
      integer nalpha_store(maxpw),jmode_store(maxpw,maxalpha)
      integer jham_store(maxpw,maxalpha)
      real*8 xweight(0:maxgrid),yweight(0:maxgrid),zweight(0:maxgrid)
      complex*16 bn1,bn2,bnorm1(maxpw,maxalpha),bnorm2(maxpw,maxalpha)
      complex*16 an2,an3,anorm2(maxpw,maxalpha),anorm3(maxpw,maxalpha)
      complex*16 dn1,dn2,dnorm1(maxpw,maxalpha),dnorm2(maxpw,maxalpha)
      complex*16 cn2,cn3,cnorm2(maxpw,maxalpha),cnorm3(maxpw,maxalpha)
      complex*16 ef(3),bf(3),efield(3),bfield(3)
      complex*16 expo,exp1,exp3,exp2p,exp2m

      logical clogical(maxham)

      integer n1vec(maxg),n2vec(maxg),iwksp(maxg),nwksp(maxg)
      integer jwork(maxham),icount(maxham),nupp,nfound
      real*8 gx,gy,gmod(maxg),gxvec(maxg),gyvec(maxg),wksp(maxg)
      real*8 akxvec(maxk),akyvec(maxk),ax(maxk),akvec(maxk)
      real*8 akx,aky,gx1,gy1,gx2,gy2,alcore(maxk),alclad1(maxk)
      real*8 alair(maxk),alclad3(maxk),altheta,theta
      real*8 alightstore(maxk,maxbands)
      complex*16 ham
      real*8 hre(maxham,maxham),umat(maxham,maxham)
      real*8 energ(maxham),eigenv(maxham,maxham),energim(maxham)
      real*8 w1(maxham),w2(maxham),wm1(2,maxham),qfacinv(maxham)
      real*8 w3(maxham),w4(maxham),wx(maxham,7)
      real*8 square_g(2),square_x(2),square_m(2),square_phi(2)
      real*8 grating_g(2),grating_x(2),grating_m(2),grating_y(2)
      real*8 triang_g(2),triang_m(2),triang_k(2)
      real*8 w1_g(2),w1_x1(2),w1_x2(2),w1_m(2)
      real*8 bands(maxk,maxbands),energ2(maxham),bandsim(maxk,maxbands)
      real*8 vgroup(maxk,maxbands),aldec(maxk,maxbands),akim
      real*8 qinvbands(maxk,maxbands),aloss(maxk,maxbands)
      real*8 ft1(maxpw,maxpw),ft2(maxpw,maxpw),ft3(maxpw,maxpw)
      real*8 ft1inv(maxpw,maxpw),ft2inv(maxpw,maxpw),ft3inv(maxpw,maxpw)
      real*8 eps1,eps2,eps_clad1,eps_core,eps_clad3
      real*8 eps1_clad1,eps2_clad1
      real*8 eps1_core ,eps2_core
      real*8 eps1_clad3,eps2_clad3
      character*4 dir
      character*10 filename
      common/dirlattice/a1,a2,ucarea
      common/reclattice/b1,b2,bzarea
      common/eps/eps1,eps2,fill
      common/length/alength1,alength2,alength3,alength4
      common/jparxy/jparxy
      common/paramf0/ag1,d,eps_core,eps_clad1,eps_clad3
      common/gvec/gxvec,gyvec
      common/matrix/umat
      common/iarray/jwork
      common/an/anorm2,anorm3,bnorm1,bnorm2,cnorm2,cnorm3,dnorm1,dnorm2
      common/ftinv/ft1inv,ft2inv,ft3inv
      common/store/om_store,q_store,chi1_store,chi3_store

      open (unit=1, file='gme.dat',status='old')
      open (unit=2, file='gme.out',status='unknown')
      open (unit=3, file='gme_gvec.out',status='unknown')
      open (unit=7, file='gme_vl.out',status='unknown')
      open (unit=9, file='gme_ll.out',status='unknown')
      open (unit=21,file='gmebis.out',status='unknown')
      open (unit=26,file='gmeloss.out',status='unknown')

      pi=4d0*atan(1d0)

      read (1,*) jlattice
      read (1,*) jbasis,alength1,alength2,alength3,alength4
      read (1,*) eps1_clad1,eps2_clad1
      read (1,*) eps1_core ,eps2_core
      read (1,*) eps1_clad3,eps2_clad3
      read (1,*) d
      read (1,*) npw,nalpha
      read (1,*) jparxy,jparkz
      read (1,*) nk
      read (1,*) jeigenv,jlossmin,jlossmax
      read (1,*) nom,error,niter
      read (1,*) factor,jshift
      read (1,*) jfreephotons
      read (1,*) jloop,varmin,varmax,nvar
      read (1,*) jfields,jkeigenv,jstate
      read (1,*) xmin,xmax,nxgrid,ymin,ymax,nygrid,zmin,zmax,nzgrid
      close (1)

      d_store=d  ! used for loop over lattice constant, when jloop=1

      if (nalpha.gt.maxalpha) stop ' nalpha>maxalpha'
      if (npw.gt.maxpw) stop ' npw>maxpw'
      if (jeigenv.lt.0.and.jeigenv.gt.1) stop ' jeigenv must be 0 or 1'
      if (jparxy.lt.-1.or.jparxy.gt.1) stop ' jparxy must be -1,0,1'
      if (jparkz.lt.-1.or.jparkz.gt.1) stop ' jparkz must be -1,0,1'
      if ((jparxy.eq.0.or.jparxy.eq.1).and.
     &    (eps1_clad1.ne.eps1_clad3.or.eps2_clad1.ne.eps2_clad3))
     &    stop ' use of jparxy requires equal upper-lower claddings'
      if (jloop.ne.0.and.jloop.ne.1) stop 'jloop must be 0 or 1'
      if (nxgrid.gt.maxgrid.or.nygrid.gt.maxgrid.or.nzgrid.gt.maxgrid) 
     &    stop 'number of grid points too large'

c
c     Calculates lattice constant when a supercell is used,
c     for the case of a 1D lattice with microcavity.
c
      if (jlattice.eq.1.and.jbasis.eq.98) then
        n1=int(alength3)
        if (n1.ne.alength3) stop ' supercell: alength3 must be integer'
        n2=n1-1
        supercellperiod=alength2+n1*alength1+n2*(1d0-alength1)
        d=d/supercellperiod
        print *, 'supercellperiod=',supercellperiod
        factor=factor/supercellperiod
      endif

c
c     defines primitive vectors of (direct) Bravais lattice
c     and unit cell area
c
      if (jlattice.eq.1) then                ! square lattice
        a1(1)=aret
        a1(2)=0d0
        a2(1)=0d0
        a2(2)=aret
      else if (jlattice.eq.2) then           ! triangular lattice
        a1(1)=aret
        a1(2)=0d0
        a2(1)=aret/2d0
        a2(2)=sqrt(3d0)/2d0*aret
      else if (jlattice.eq.4) then           ! W1 waveguide
        a1(1)=aret
        a1(2)=0d0
        a2(1)=0d0
        a2(2)=aret*(alength2+alength3)*sqrt(3d0)
      else   
        stop ' jlattice must be 1, 2 or 4'
      endif
      ucarea=abs(a1(1)*a2(2)-a1(2)*a2(1))
c
c     defines primitive vectors of reciprocal lattice and unit cell area
c
      b1(1)= 2d0*pi/ucarea*a2(2)
      b1(2)=-2d0*pi/ucarea*a2(1)
      b2(1)=-2d0*pi/ucarea*a1(2)
      b2(2)= 2d0*pi/ucarea*a1(1)
      bzarea=4d0*pi*pi/ucarea

c
c     calculates reciprocal lattice vectors
c
      bl1=sqrt(b1(1)*b1(1)+b1(2)*b1(2))
      bl2=sqrt(b2(1)*b2(1)+b2(2)*b2(2))
      if (bl1.le.bl2) then
        n2max=50
        n1max=n2max*bl2/bl1
      else
        n1max=50
        n2max=n1max*bl1/bl2
      endif
      if (jlattice.eq.1.and.jbasis.gt.90) then
        n1max=500
        n2max=0
      endif

      j=0
      print *, 'WARNING: WHEN JBASIS=99 ONLY REC. LATT. VECTORS ALONG X'
      do n1=-n1max,n1max
      do n2=-n2max,n2max
        j=j+1
        if (j.gt.maxg) stop ' j>maxg'
        gx=n1*b1(1)+n2*b2(1)
        gy=n1*b1(2)+n2*b2(2)
        gmod(j)=sqrt(gx*gx+gy*gy)
        n1vec(j)=n1
        n2vec(j)=n2
        gxvec(j)=gx
        gyvec(j)=gy
      enddo
      enddo
      nrec=j
      call sort5 (nrec,gmod,gxvec,gyvec,n1vec,n2vec,wksp,iwksp,nwksp)
      do j=1,10*npw
        write (3,12) j,n1vec(j),n2vec(j),gxvec(j),gyvec(j),gmod(j)
      enddo
12    format (1x,i4,4x,i3,2x,i3,4x,f10.4,2x,f10.4,4x,f14.4)

c
c     loop over variable VAR (dielectric constant or hole radius or...)
c
      if (nvar.ne.0) delvar=(varmax-varmin)/float(nvar)
      if (jloop.eq.0.and.nvar.ne.0) stop 'no loop over VAR if JLOOP=0'
      do 9995 jvar=0,nvar
      var=varmin+jvar*delvar

      if (jloop.eq.1) then
         print *, 'loop variable is alength1'
         alength1=var
!         alength3=var
!         alength2=var
!         alattice=var     ! lattice constant in nm
!         d=d_store/alattice
!         factor=197.33*2d0*pi/alattice
         if (jvar+1.lt.10) write (filename,9991) jvar+1
         if (jvar+1.ge.10) write (filename,9992) jvar+1
9991     format('gme00',i1,'.out')
9992     format('gme0',i2,'.out')
         close (2)
         print *, 'writing output on file ',filename
         open (unit=2,file=filename,status='unknown')
      endif

c
c     calculates matrix with Fourier transform of dielectric constant
c
      do 120 j1=1,npw
      do 120 j2=1,npw
        gx1=gxvec(j1)
        gy1=gyvec(j1)
        gx2=gxvec(j2)
        gy2=gyvec(j2)
c
c       Fourier transform of dielectric constant - UPPER CLADDING
c
        eps1=eps1_clad1
        eps2=eps2_clad1
        if (jlattice.eq.1) then       ! square lattice
          if (jbasis.eq.1) then           ! circular rods
            ft1(j1,j2)=ft11(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.2) then      ! square rods, non rotated
            ft1(j1,j2)=ft12(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.3) then      ! square rods, 45 rotated
            ft1(j1,j2)=ft13(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.98) then     ! 1D stripes with microacvity
            ft1(j1,j2)=ft98(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.99) then     ! 1D stripes
            ft1(j1,j2)=ft99(gx1-gx2,gy1-gy2)
          else
            stop ' when jlattice=1 jbasis must be 1,2,3, 98, 99'
          endif
        else if (jlattice.eq.2) then
          if (jbasis.eq.1) then           ! circular rods
            ft1(j1,j2)=ft11(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.2) then      ! circular rods, graphite
            ft1(j1,j2)=ft22(gx1-gx2,gy1-gy2)
          else
            stop ' when jlattice=2 jbasis must be 1,2'
          endif
        else if (jlattice.eq.4) then
          if (jbasis.eq.0) then     ! bulk with W1 supercell
            ft1(j1,j2)=ft40(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.1) then      ! W1 waveguide with supercell
            ft1(j1,j2)=ft41(gx1-gx2,gy1-gy2)
          else
            stop ' when jlattice=4 jbasis must be 0,1'
          endif
        endif
c
c       Fourier transform of dielectric constant - CORE
c
        eps1=eps1_core
        eps2=eps2_core
        if (jlattice.eq.1) then       ! square lattice
          if (jbasis.eq.1) then           ! circular rods
            ft2(j1,j2)=ft11(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.2) then      ! square rods, non rotated
            ft2(j1,j2)=ft12(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.3) then      ! square rods, 45 rotated
            ft2(j1,j2)=ft13(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.98) then     ! 1D stripes with microacvity
            ft2(j1,j2)=ft98(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.99) then     ! 1D stripes
            ft2(j1,j2)=ft99(gx1-gx2,gy1-gy2)
          else
            stop ' when jlattice=1 jbasis must be 1,2,3,98,99'
          endif
        else if (jlattice.eq.2) then
          if (jbasis.eq.1) then           ! circular rods
            ft2(j1,j2)=ft11(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.2) then      ! circular rods, graphite
            ft2(j1,j2)=ft22(gx1-gx2,gy1-gy2)
          endif
        else if (jlattice.eq.4) then
          if (jbasis.eq.0) then     ! bulk with W1 supercell
            ft2(j1,j2)=ft40(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.1) then      ! W1 waveguide
            ft2(j1,j2)=ft41(gx1-gx2,gy1-gy2)
          else
            stop ' when jlattice=4 jbasis must be 0,1'
          endif
        endif
c
c       Fourier transform of dielectric constant - LOWER CLADDING
c
        eps1=eps1_clad3
        eps2=eps2_clad3
        if (jlattice.eq.1) then       ! square lattice
          if (jbasis.eq.1) then           ! circular rods
            ft3(j1,j2)=ft11(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.2) then      ! square rods, non rotated
            ft3(j1,j2)=ft12(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.3) then      ! square rods, 45 rotated
            ft3(j1,j2)=ft13(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.98) then     ! 1D stripes with microacvity
            ft3(j1,j2)=ft98(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.99) then     ! 1D stripes
            ft3(j1,j2)=ft99(gx1-gx2,gy1-gy2)
          else
            stop ' when jlattice=1 jbasis must be 1,2,3,98, 99'
          endif
        else if (jlattice.eq.2) then
          if (jbasis.eq.1) then           ! circular rods
            ft3(j1,j2)=ft11(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.2) then      ! circular rods, graphite
            ft3(j1,j2)=ft22(gx1-gx2,gy1-gy2)
          endif
        else if (jlattice.eq.4) then
          if (jbasis.eq.0) then     ! bulk with W1 supercell, my routine
            ft3(j1,j2)=ft40(gx1-gx2,gy1-gy2)
          else if (jbasis.eq.1) then      ! W1 waveguide, routine Agio
            ft3(j1,j2)=ft41(gx1-gx2,gy1-gy2)
          else
            stop ' when jlattice=4 jbasis must be 0,1'
          endif
        endif
        if (jfreephotons.eq.1) then
          ft1(j1,j2)=delta_kronecker(j1,j2)*ft1(1,1)
          ft2(j1,j2)=delta_kronecker(j1,j2)*ft2(1,1)
          ft3(j1,j2)=delta_kronecker(j1,j2)*ft3(1,1)
        endif
120   continue
      write (6,121) fill
121   format (1x,'filling fraction=',f12.6)
c
c     Fourier transform of inverse dielectric constant (Ho's method)
c     Linpack routines are used (http://www.netlib.org/linpack)
c
      print *, ' starts inversion of dielectric matrix'

      call savemt (ft1,ft1inv,maxpw,npw)
      call dpoco (ft1inv,maxpw,npw,rcond,w1,ifail)
      if (ifail.gt.0) stop ' after DPOCO: ifail>0'
      if (abs(rcond).lt.1d-14) stop ' after DPOCO: rcond too small'
      job=1
      call dpodi (ft1inv,maxpw,npw,dummy,job)
      do j1=2,npw
       do j2=1,j1-1
         ft1inv(j1,j2)=ft1inv(j2,j1)
       enddo
      enddo
      call checkinv (ft1,ft1inv,maxpw,npw)
      call checksym (ft1inv,maxpw,npw)
      print *, ' ends inversion of dielectric matrix'

      call savemt (ft2,ft2inv,maxpw,npw)
      call dpoco (ft2inv,maxpw,npw,rcond,w1,ifail)
      if (ifail.gt.0) stop ' after DPOCO: ifail>0'
      if (abs(rcond).lt.1d-14) stop ' after DPOCO: rcond too small'
      job=1
      call dpodi (ft2inv,maxpw,npw,dummy,job)
      do j1=2,npw
       do j2=1,j1-1
         ft2inv(j1,j2)=ft2inv(j2,j1)
       enddo
      enddo
      call checksym (ft2inv,maxpw,npw)
      call checkinv (ft2,ft2inv,maxpw,npw)

      call savemt (ft3,ft3inv,maxpw,npw)
      call dpoco (ft3inv,maxpw,npw,rcond,w1,ifail)
      if (ifail.gt.0) stop ' after DPOCO: ifail>0'
      if (abs(rcond).lt.1d-14) stop ' after DPOCO: rcond too small'
      job=1
      call dpodi (ft3inv,maxpw,npw,dummy,job)
      do j1=2,npw
       do j2=1,j1-1
         ft3inv(j1,j2)=ft3inv(j2,j1)
       enddo
      enddo
      call checksym (ft3inv,maxpw,npw)
      call checkinv (ft3,ft3inv,maxpw,npw)

      print *,' components (0,0) of dielectric tensor:'
      write (6,122) ft1(1,1),ft2(1,1),ft3(1,1)
      print *,' inverse components (0,0) of inverse dielectric tensor:'
      write (6,122) 1d0/ft1inv(1,1),1d0/ft2inv(1,1),1d0/ft3inv(1,1)
122   format(1x,' clad1=',f12.6,'  core=',f12.6,'  clad3=',f12.6)

!      print *, ' dielectric tensor:'
!      call wrmat (ft1,maxpw,npw,1d0,0d0)
!      call wrmat (ft2,maxpw,npw,1d0,0d0)
!      call wrmat (ft3,maxpw,npw,1d0,0d0)
!      print *, ' inverse dielectric tensor:'
!      call wrmat (ft1inv,maxpw,npw,1d0,0d0)
!      call wrmat (ft2inv,maxpw,npw,1d0,0d0)
!      call wrmat (ft3inv,maxpw,npw,1d0,0d0)

      print *, ' ends inversion of dielectric matrix'

c
c     prepares k-vectors for calculating photonic bands (akxvec,akyvec)
c     and x-coordinate vector for plotting (ax)
c
      axmult=1d0
      axsum=0d0
      do jk=1,maxk
        ax(jk)=0d0
      enddo
      if (jlattice.eq.1) then

        if (jbasis.eq.98.or.jbasis.eq.99) then

          call grating_points (grating_g,grating_x,grating_y,grating_m)


!         nktot=5*nk+1  
!         call kline(grating_g,grating_x,akxvec,akyvec,ax,maxk,nk,1)
!         call kline(grating_x,grating_m,akxvec,akyvec,ax,maxk,nk,1+nk)
!         call kline(grating_m,grating_g,akxvec,akyvec,ax,maxk,nk,1+2*nk)
!         call kline(grating_g,grating_y,akxvec,akyvec,ax,maxk,nk,1+3*nk)
!         call kline(grating_y,grating_m,akxvec,akyvec,ax,maxk,nk,1+4*nk)

         nktot=nk+1  
         dir(1:4)='xgam'
         call kline(grating_g,grating_x,akxvec,akyvec,ax,maxk,nk,1)

!         grating_y(2)=grating_y(2)*5.
!         nktot=nk+1  
!         dir(1:4)='ygam'
!         call kline(grating_g,grating_y,akxvec,akyvec,ax,maxk,nk,1)

        else

         call square_points (square_g,square_x,square_m)

!         nktot=nk+1  
!         dir(1:4)='xgam'
!         call kline (square_g,square_x,akxvec,akyvec,ax,maxk,nk,1)


!         nktot=nk+1
!         call kline (square_x,square_g,akxvec,akyvec,ax,maxk,nk,1)
!         dir(1:4)='xgam'
!         axmult=1d0/(1d0+sqrt(2d0))
!         axsum=0d0

!         nktot=nk+1
!         call kline (square_g,square_m,akxvec,akyvec,ax,maxk,nk,1)
!         dir(1:4)='gamm'
!         axmult=sqrt(2d0)/(1d0+sqrt(2d0))
!         axsum=1d0/(1d0+sqrt(2d0))


!         nktot=2*nk+1
!         call kline (square_x,square_g,akxvec,akyvec,ax,maxk,nk,1)
!         call kline (square_g,square_m,akxvec,akyvec,ax,maxk,nk,1+nk)

!         nkh=nk/2
!         nktot=2*nk+nkh+1
!         call kline (square_x,square_g,akxvec,akyvec,ax,maxk,nk,1)
!         call kline (square_g,square_m,akxvec,akyvec,ax,maxk,nk,1+nk)
!         call kline (square_m,square_x,akxvec,akyvec,ax,maxk,nkh,1+2*nk)

        nktot=3*nk+1
        call kline (square_g,square_x,akxvec,akyvec,ax,maxk,nk,1)
        call kline (square_x,square_m,akxvec,akyvec,ax,maxk,nk,1+nk)
        call kline (square_m,square_g,akxvec,akyvec,ax,maxk,nk,1+2*nk)

        endif
      else if (jlattice.eq.2) then
        call triang_points (triang_g,triang_m,triang_k)

!        nktot=nk+1
!        call kline (triang_m,triang_g,akxvec,akyvec,ax,maxk,nk,1)
!        dir(1:4)='mgam'
!        axmult=1d0/(1d0+2d0/sqrt(3d0))
!        axsum=0d0

!        nktot=nk+1 
!        call kline (triang_g,triang_k,akxvec,akyvec,ax,maxk,nk,1)
!        dir (1:4)='gamk'
!        axmult=2d0/sqrt(3d0)/(1d0+2d0/sqrt(3d0))
!        axsum=1d0/(1d0+2d0/sqrt(3d0))

!        nktot=2*nk+1
!        call kline (triang_m,triang_g,akxvec,akyvec,ax,maxk,nk,1)
!        call kline (triang_g,triang_k,akxvec,akyvec,ax,maxk,nk,1+nk)

!        nkh=nk/2
!        nktot=2*nk+nkh+1
!        call kline (triang_m,triang_g,akxvec,akyvec,ax,maxk,nk,1)
!        call kline (triang_g,triang_k,akxvec,akyvec,ax,maxk,nk,1+nk)
!        call kline (triang_k,triang_m,akxvec,akyvec,ax,maxk,nkh,1+2*nk)

        nktot=3*nk+1
        call kline (triang_g,triang_m,akxvec,akyvec,ax,maxk,nk,1)
        call kline (triang_m,triang_k,akxvec,akyvec,ax,maxk,nk,1+nk)
        call kline (triang_k,triang_g,akxvec,akyvec,ax,maxk,nk,1+2*nk)

!        call kline (triang_g,triang_k,akxvec,akyvec,ax,maxk,nk,1)
!        call kline (triang_k,triang_m,akxvec,akyvec,ax,maxk,nk,1+nk)
!        call kline (triang_m,triang_g,akxvec,akyvec,ax,maxk,nk,1+2*nk)

      else if (jlattice.eq.4) then  

!        call square_points (square_g,square_x,square_m) 
!        call kline (square_g,square_x,akxvec,akyvec,ax,maxk,nk,1)

        call w1_points (w1_g,w1_x1,w1_x2,w1_m)

        nktot=nk+1
        dir(1:4)='xgam'
        call kline (w1_g,w1_x1,akxvec,akyvec,ax,maxk,nk,1)

!        nktot=2*nk+1
!        call kline (w1_x2,w1_g,akxvec,akyvec,ax,maxk,nk,1)
!        call kline (w1_g,w1_x1,akxvec,akyvec,ax,maxk,nk,1+nk)


!        nkh=nk/2
!        nktot=2*nk+2*nkh+1
!        call kline (w1_g,w1_x1,akxvec,akyvec,ax,maxk,nk,1)
!        call kline (w1_x1,w1_m,akxvec,akyvec,ax,maxk,nkh,1+  nk)
!        call kline (w1_m,w1_x2,akxvec,akyvec,ax,maxk,nk ,1+  nk+nkh)
!        call kline (w1_x2,w1_g,akxvec,akyvec,ax,maxk,nkh,1+2*nk+nkh)

!        nktot=4*nk+1
!        call kline (w1_g,w1_x1,akxvec,akyvec,ax,maxk,nk,1)
!        call kline (w1_x1,w1_m,akxvec,akyvec,ax,maxk,nk,1+nk)
!        call kline (w1_m,w1_x2,akxvec,akyvec,ax,maxk,nk,1+2*nk)
!        call kline (w1_x2,w1_g,akxvec,akyvec,ax,maxk,nk,1+3*nk)


      endif

      if (nktot.gt.maxk) stop ' nktot>maxk'
      do jk=1,nktot
        if (abs(akxvec(jk)).lt.upper) akxvec(jk)=akxvec(jk)+upper
        if (abs(akyvec(jk)).lt.upper) akyvec(jk)=akyvec(jk)+upper
        if (nk.ne.0) ax(jk)=ax(jk)/ax(nktot)*axmult+axsum
        akvec(jk)=sqrt(akxvec(jk)*akxvec(jk)+akyvec(jk)*akyvec(jk))
      enddo

22    format (1x,i4,4x,3(f12.6,2x))


c
c     redefinition of dielectric constants as average value 
c     (diagonal component of dielectric tensor)
c

      eps_clad1=ft1(1,1)
      eps_core =ft2(1,1)
      eps_clad3=ft3(1,1)

!      eps_core=var ! for convergence test as a function of effective epsilon

      print *, ' DIEL. CONSTANT OF UPPER CLADDING DEFINED AS',eps_clad1
      print *, ' DIEL. CONSTANT OF       CORE     DEFINED AS',eps_core
      print *, ' DIEL. CONSTANT OF LOWER CLADDING DEFINED AS',eps_clad3
c
c     starts loop over wavevectors
c
      do 200 jk=1,nktot
!        if (akxvec(jk).gt.0.7d0) goto 200 ! ociu
!        if (jk.eq.1.or.jk.eq.nktot) goto 200 !ociu
        akx=akxvec(jk)
        aky=akyvec(jk)
c
c       starts loop over plane waves to calculate basis of waveguide modes
c
        do 210 j1=1,npw
          agx1=akx+gxvec(j1)
          agy1=aky+gyvec(j1)
          ag1 =sqrt(agx1*agx1+agy1*agy1)
c
c         calculates guided modes of planar waveguide at wavevector ag1
c
          om_core =ag1/sqrt(eps_core )*(1d0+upper)
          om_clad1=ag1/sqrt(eps_clad1)*(1d0-upper)
          om_clad3=ag1/sqrt(eps_clad3)*(1d0-upper)
          om_clad=min(om_clad1,om_clad3)
          call setacc (nalpha,nalpha1,om_core,om_clad,nom,
     &                 neffet,fwaveguide,omlow,omupp)
          do 220 jfun=1,nalpha1
            call zero (omlow(jfun),omupp(jfun),omega1,
     &                 error,diff,niter,neffet,fwaveguide,flag)
            q     =sqrt(eps_core*omega1*omega1-ag1*ag1)
            chi1  =sqrt(ag1*ag1-eps_clad1*omega1*omega1)
            chi3  =sqrt(ag1*ag1-eps_clad3*omega1*omega1)
            om_store  (j1,jfun)=omega1
            q_store   (j1,jfun)=q
            chi1_store(j1,jfun)=chi1
            chi3_store(j1,jfun)=chi3
            aacos=cos(q*d)
            aasin=sin(q*d)
            funte=q*(chi1+chi3)*aacos+(chi1*chi3-q*q)*aasin
            funtm=q/eps_core*(chi1/eps_clad1+chi3/eps_clad3)*aacos+
     &           (chi1*chi3/eps_clad1/eps_clad3-q*q/eps_core**2)*aasin
            if (abs(funte).lt.abs(funtm)) then
              jmode_store(j1,jfun)=0
            else
              jmode_store(j1,jfun)=1
            endif
220       continue
          nalpha_store(j1)=nalpha1
c
c         constructs coefficients of basis at given wavevector
c
!          write (18,228) agx1,agy1,nalpha1
!          write (19,227) ag1,(om_store(j1,jal1),jal1=1,nalpha1)
!          write (20,226) ag1,(jmode_store(j1,jal1),jal1=1,nalpha1)
227       format(1x,f12.6,2x,8(1x,f12.6))
226       format(1x,f12.6,2x,8(1x,i3))
228       format(/,' wavevector=',f12.6,1x,f12.6,' # of modes=',i2,/)
229       format('   jal1=',i2,' jmode=',i2,' frequency=',f12.6)
          do 230 jal1=1,nalpha1
            jmode1=jmode_store(j1,jal1)
            omega1=om_store (j1,jal1)
!            write (18,229) jal1,jmode1,omega1*aret/2d0/pi*factor
            q     =q_store  (j1,jal1)
            chi1  =chi1_store(j1,jal1)
            chi3  =chi3_store(j1,jal1)
            if (jmode1.eq.0) then       ! TE mode
              bn1=1d0
              an2=(q-i*chi1)*exp( i*q*d/2d0)/2d0/q
              bn2=(q+i*chi1)*exp(-i*q*d/2d0)/2d0/q
              an3=(q*(chi3-chi1)*cos(q*d)+(q*q+chi3*chi1)*sin(q*d))
     &           /2d0/chi3/q
              znorm=dconjg(bn1)*bn1*(chi1*chi1+ag1*ag1)/2d0/chi1
     &             +dconjg(an3)*an3*(chi3*chi3+ag1*ag1)/2d0/chi3
     &             +d*( ( q*q+ag1*ag1)*(dconjg(an2)*an2+dconjg(bn2)*bn2)
     &             +(-q*q+ag1*ag1)*(dconjg(an2)*bn2+dconjg(bn2)*an2)
     &             *spher0(q*d) )
              bnorm1(j1,jal1)=bn1/sqrt(znorm)
              anorm2(j1,jal1)=an2/sqrt(znorm)
              bnorm2(j1,jal1)=bn2/sqrt(znorm)
              anorm3(j1,jal1)=an3/sqrt(znorm)
!              write (18,231) bn1,an2,bn2,an3
            else if (jmode1.eq.1) then       ! TM mode
              dn1=1d0
              cn2=(q/eps_core-i*chi1/eps_clad1)*exp(i*q*d/2d0)
     &            *eps_core/2d0/q
              dn2=(q/eps_core+i*chi1/eps_clad1)*exp(-i*q*d/2d0)
     &            *eps_core/2d0/q
              cn3=( q/eps_core*(chi3/eps_clad3-chi1/eps_clad1)*cos(q*d)
     &           +(q*q/eps_core/eps_core+chi3/eps_clad3*chi1/eps_clad1)
     &           *sin(q*d) ) * eps_clad3*eps_core/2d0/chi3/q
              znorm=dconjg(dn1)*dn1/2d0/chi1+dconjg(cn3)*cn3/2d0/chi3
     &             +d*( dconjg(cn2)*cn2+dconjg(dn2)*dn2
     &             +   (dconjg(cn2)*dn2+dconjg(dn2)*cn2)*spher0(q*d) )
              dnorm1(j1,jal1)=dn1/sqrt(znorm)
              cnorm2(j1,jal1)=cn2/sqrt(znorm)
              dnorm2(j1,jal1)=dn2/sqrt(znorm)
              cnorm3(j1,jal1)=cn3/sqrt(znorm)
!              write (18,231) dn1,cn2,dn2,cn3
231           format(4(1x,e9.3,',',e9.3))
            endif

230       continue
210     continue


c
c       calculates dimension of Hamiltonian for the given Bloch vector
c
        nham=0
        do j1=1,npw
          nham=nham+nalpha_store(j1)        
        enddo
        write (6,212) nham
        nham_store=nham
212     format(' dimension of hamiltonian=',i4)


c
c       starts construction of Hamiltonian and overlap matrices
c
        call initialize (hre,maxham,nham)

        pivot=0d0
        jham1=0
        do 240 j1=1,npw
        do 240 jal1=1,nalpha_store(j1)
         jham1=jham1+1
         jham_store(j1,jal1)=jham1
         jham2=0
         do 240 j2=1,npw
         do 240 jal2=1,nalpha_store(j2)
          jham2=jham2+1
          jmode1=jmode_store(j1,jal1)
          jmode2=jmode_store(j2,jal2)
          agx1=akx+gxvec(j1)
          agy1=aky+gyvec(j1)
          ag1 =sqrt(agx1*agx1+agy1*agy1)
          agx2=akx+gxvec(j2)
          agy2=aky+gyvec(j2)
          ag2 =sqrt(agx2*agx2+agy2*agy2)
          omega1=om_store(j1,jal1)
          omega2=om_store(j2,jal2)
          q =q_store(j1,jal1)
          qp=q_store(j2,jal2)
          chi1 =chi1_store(j1,jal1)
          chi1p=chi1_store(j2,jal2)
          chi3 =chi3_store(j1,jal1)
          chi3p=chi3_store(j2,jal2)
          pol1(1)= agx1/ag1
          pol1(2)= agy1/ag1
          qol1(1)=-agy1/ag1
          qol1(2)= agx1/ag1
          pol2(1)= agx2/ag2
          pol2(2)= agy2/ag2
          qol2(1)=-agy2/ag2
          qol2(2)= agx2/ag2
          if      (jmode1.eq.0.and.jmode2.eq.0)  then ! TE-TE
            ham=omega1*omega1*omega2*omega2*scal(qol1,qol2,2)
     &       * ( eps_clad1*eps_clad1*ft1inv(j1,j2)
     &           *dconjg(bnorm1(j1,jal1))*bnorm1(j2,jal2)/(chi1+chi1p)

     &          +eps_clad3*eps_clad3*ft3inv(j1,j2)
     &           *dconjg(anorm3(j1,jal1))*anorm3(j2,jal2)/(chi3+chi3p)

     &          +eps_core*eps_core*ft2inv(j1,j2)*(
     &           (dconjg(anorm2(j1,jal1))*anorm2(j2,jal2)
     &           +dconjg(bnorm2(j1,jal1))*bnorm2(j2,jal2))
     &           *d*spher0((q-qp)*d/2d0)
     &          +(dconjg(anorm2(j1,jal1))*bnorm2(j2,jal2)
     &           +dconjg(bnorm2(j1,jal1))*anorm2(j2,jal2))
     &           *d*spher0((q+qp)*d/2d0) ) )
          else if (jmode1.eq.1.and.jmode2.eq.1)  then ! TM-TM
            ham=
     &        ft1inv(j1,j2)*dconjg(dnorm1(j1,jal1))*dnorm1(j2,jal2)
     &        *(chi1*chi1p*scal(pol1,pol2,2)+ag1*ag2)/(chi1+chi1p)
     &       +ft3inv(j1,j2)*dconjg(cnorm3(j1,jal1))*cnorm3(j2,jal2)
     &        *(chi3*chi3p*scal(pol1,pol2,2)+ag1*ag2)/(chi3+chi3p)
     &       +ft2inv(j1,j2)*(
     &        (dconjg(cnorm2(j1,jal1))*cnorm2(j2,jal2)
     &        +dconjg(dnorm2(j1,jal1))*dnorm2(j2,jal2))
     &       *( q*qp*scal(pol1,pol2,2)+ag1*ag2)*d*spher0((q-qp)*d/2d0)
     &       +(dconjg(cnorm2(j1,jal1))*dnorm2(j2,jal2)
     &        +dconjg(dnorm2(j1,jal1))*cnorm2(j2,jal2))
     &       *(-q*qp*scal(pol1,pol2,2)+ag1*ag2)*d*spher0((q+qp)*d/2d0))
          else if (jmode1.eq.0.and.jmode2.eq.1)  then ! TE-TM
            ham=omega1*omega1*scal(qol1,pol2,2)* (
     &       -eps_clad1*ft1inv(j1,j2)*chi1p/(chi1+chi1p)
     &       *dconjg(bnorm1(j1,jal1))*dnorm1(j2,jal2)
     &       +eps_clad3*ft3inv(j1,j2)*chi3p/(chi3+chi3p)
     &       *dconjg(anorm3(j1,jal1))*cnorm3(j2,jal2)
     &       +i*eps_core*ft2inv(j1,j2)*qp* (
     &        (-dconjg(anorm2(j1,jal1))*cnorm2(j2,jal2)
     &         +dconjg(bnorm2(j1,jal1))*dnorm2(j2,jal2))
     &        *d*spher0((q-qp)*d/2d0)
     &       +( dconjg(anorm2(j1,jal1))*dnorm2(j2,jal2)
     &         -dconjg(bnorm2(j1,jal1))*cnorm2(j2,jal2))
     &        *d*spher0((q+qp)*d/2d0) ) )
          else if (jmode1.eq.1.and.jmode2.eq.0)  then ! TM-TE
            ham=omega2*omega2*scal(pol1,qol2,2)* (
     &       -eps_clad1*ft1inv(j1,j2)*chi1/(chi1+chi1p)
     &       *dconjg(dnorm1(j1,jal1))*bnorm1(j2,jal2)
     &       +eps_clad3*ft3inv(j1,j2)*chi3/(chi3+chi3p)
     &       *dconjg(cnorm3(j1,jal1))*anorm3(j2,jal2)
     &       -i*eps_core*ft2inv(j1,j2)*q* (
     &        (-dconjg(cnorm2(j1,jal1))*anorm2(j2,jal2)
     &         +dconjg(dnorm2(j1,jal1))*bnorm2(j2,jal2))
     &        *d*spher0((q-qp)*d/2d0)
     &       +( dconjg(dnorm2(j1,jal1))*anorm2(j2,jal2)
     &         -dconjg(cnorm2(j1,jal1))*bnorm2(j2,jal2))
     &        *d*spher0((q+qp)*d/2d0) ) )
          endif

          hre(jham1,jham2)=dble(ham)
          rivale=dimag(ham)
          pivot=max(pivot,rivale)

240     continue

        if (pivot.gt.1d-10) then
          print *, ' pivot of Him=',pivot
          stop ' Hamiltonian matrix is not real: something went wrong'
        endif

c
c       checks hamiltonian is hermitian
c
        call checksym (hre,maxham,nham)
!        call repulisti (hre,maxham,nham)
!        call wrmat (hre,maxham,nham,1d0,0d0)

!        print *, 'before first call to checksym: nham=',nham
!        call wrmat (hre,maxham,nham,1d0,0d0)
!        call checksym (hre,maxham,nham)
        if (jparkz.eq.0.or.jparkz.eq.1) call decouwga(akx,aky,npw,hre,
     &    jlattice,jparkz,dir,nham,nalpha_store,jmode_store,jham_store)
!        print *, 'before second call to checksym: nham=',nham
!        call wrmat (hre,maxham,nham,1d0,0d0)
!        call checksym (hre,maxham,nham)

        write (6,101) jk,akx,aky,nham
101     format (1x,i3,2x,f12.6,2x,f12.6,2x,i4)
        if (nham.eq.0) goto 200
        call checksym (hre,maxham,nham)

c
c       solves eigenvalue problem
c
!        print *, ' starts diagonalization of hamiltonian'
        ifail=0
        call rs (maxham,nham,hre,energ2,jeigenv,eigenv,w1,w2,ifail)
c
c       restores eigenvectors in original basis, if parity symmetry was used
c
        if (jeigenv.eq.1.and.(jparkz.eq.0.or.jparkz.eq.1))
     &    call restore (eigenv,nham,nham_store)

!        call wrvec (energ2,maxham,nham,1d0,0d0)
!        write (8,101) jk,akx,aky,nham
!        call wrvec (energ2,maxham,nham,1d0,0d0)
!        call wrmat (eigenv,maxham,nham,1d0,0d0)
!
!        print *, ' ends diagonalization of hamiltonian'
c
c       checks energies are positive, rescales in dimensionless units
c
        alcore (jk)=akvec(jk)/2d0/pi/sqrt(eps_core) *factor
        alclad1(jk)=akvec(jk)/2d0/pi/sqrt(eps_clad1)*factor
        alclad3(jk)=akvec(jk)/2d0/pi/sqrt(eps_clad3)*factor
        alair (jk) =akvec(jk)/2d0/pi                *factor
c
c       cladding light line in all Brillouin zones (for W1 loss)
c
        do j1=1,maxbands 
          alight=sqrt((akx+gxvec(j1))**2+(aky+gyvec(j1))**2)
     &          /2d0/pi/sqrt(eps_clad1)*factor
          alightstore(jk,j1)=alight
!          write (34,202) akx/pi,aky/pi,alight
202       format (1x,3(e12.6,2x))
        enddo
        do j1=1,nham
          if (energ2(j1).ge.0) then
            energ(j1)=sqrt(energ2(j1))*aret/2d0/pi*factor
!            energ(j1)=sqrt(energ2(j1))*aret
          else if (-1d-8.lt.energ2(j1)) then
            energ(j1)=0d0
          else
            energ(j1)=-99.
          endif
        enddo

        nbands=min(maxbands,nham) ! number of bands to be written
c
c       Calculates and writes field components.
c       Starts calculation of electric and magnetic fields in a cubic grid
c
        if (jfields.eq.1.and.jeigenv.eq.1.and.jk.eq.jkeigenv) then

c
c         calculates normalization of electric field (analytic)
c
          elnorm=0d0
          do 650 j1=1,npw
          do 650 jal1=1,nalpha_store(j1)
            jham1 =jham_store (j1,jal1)
            om    =om_store(j1,jal1)
            elnorm=elnorm+eigenv(jham1,jstate)**2*om*om
650       continue
          elnorm=elnorm/(energ(jstate)/aret*2d0*pi/factor)**2

          if (nxgrid.ne.0) delx=(xmax-xmin)/float(nxgrid)
          if (nygrid.ne.0) dely=(ymax-ymin)/float(nygrid)
          if (nzgrid.ne.0) delz=(zmax-zmin)/float(nzgrid)

c
c         defines weights for numerical integration of electric field
c

          do jxgrid=0,nxgrid
            if (jxgrid.eq.0.or.jxgrid.eq.nxgrid) then
              xweight(jxgrid)=0.5d0*delx
            else
              xweight(jxgrid)=delx
            endif
          enddo
          do jygrid=0,nygrid
            if (jygrid.eq.0.or.jygrid.eq.nygrid) then
              yweight(jygrid)=0.5d0*dely
            else
              yweight(jygrid)=dely
            endif
          enddo
          do jzgrid=0,nzgrid
            if (jzgrid.eq.0.or.jzgrid.eq.nzgrid) then
              zweight(jzgrid)=0.5d0*delz
            else
              zweight(jzgrid)=delz
            endif
          enddo

          do 630 jxgrid=0,nxgrid
          do 631 jygrid=0,nygrid
          do 632 jzgrid=0,nzgrid
            xx=xmin+delx*jxgrid
            yy=ymin+dely*jygrid
            zz=zmin+delz*jzgrid
c
c           WRONG FOR ELECTRIC FIELD
c           approximates dielectric constant with average value for E calculation
c           Procedure to be implemented: calculate it exactly
c           for specific lattices.
c
            if (zz.lt.-d/2d0) epsilon=eps_clad1
            if (zz.ge.-d/2d0.and.zz.le.d/2d0) epsilon=eps_core
            if (zz.gt. d/2d0) epsilon=eps_clad3

            do jcomp=1,3
              efield(jcomp)=(0d0,0d0)
              bfield(jcomp)=(0d0,0d0)
            enddo
c
c           starts sum over basis states
c           bf(1),bf(2),bf(3): coefficients of H-field
c           ef(1),ef(2),ef(3): coefficients of nabla x H-field,
c           later multiplied by ic/omega to get electric field
c
            eps_realspace=0d0
            do 640 j1=1,npw
             expo=exp(i*(gxvec(j1)*xx+gyvec(j1)*yy))
             if (zz.lt.-d/2d0) then                       ! medium 1
               eps_realspace=eps_realspace+ft1(j1,1)*expo
             else if (zz.ge.-d/2d0.and.zz.le.d/2d0) then  ! medium 2
               eps_realspace=eps_realspace+ft2(j1,1)*expo
             else if (zz.gt.d/2d0) then                   ! medium 3
               eps_realspace=eps_realspace+ft3(j1,1)*expo
             endif
             do 640 jal1=1,nalpha_store(j1)
              jham1 =jham_store (j1,jal1)
              jmode1=jmode_store(j1,jal1)
              agx1=akx+gxvec(j1)
              agy1=aky+gyvec(j1)
              ag1 =sqrt(agx1*agx1+agy1*agy1)
              pol1(1)= agx1/ag1
              pol1(2)= agy1/ag1
              qol1(1)=-agy1/ag1
              qol1(2)= agx1/ag1
              expo=exp(i*(agx1*xx+agy1*yy))
              om  =om_store(j1,jal1)
              chi1=chi1_store(j1,jal1)
              q   =q_store(j1,jal1)
              chi3=chi3_store(j1,jal1)
              if (jmode1.eq.0) then        ! TE-polarized basis state
                bn1=bnorm1(j1,jal1)
                bn2=bnorm2(j1,jal1)
                an2=anorm2(j1,jal1)
                an3=anorm3(j1,jal1)
                if (zz.lt.-d/2d0) then                       ! medium 1
                  exp1=exp(chi1*(zz+d/2d0))
                  bf(1)=-bn1*chi1*pol1(1)*exp1
                  bf(2)=-bn1*chi1*pol1(2)*exp1
                  bf(3)= bn1*i*ag1       *exp1
                  ef(1)= om*om*qol1(1)*eps_clad1*bn1*exp1
                  ef(2)= om*om*qol1(2)*eps_clad1*bn1*exp1
                  ef(3)= (0d0,0d0)
                else if (zz.ge.-d/2d0.and.zz.le.d/2d0) then  ! medium 2
                  exp2p=exp( i*q*zz)
                  exp2m=exp(-i*q*zz)
                  bf(1)= i*q*pol1(1)*(-an2*exp2p+bn2*exp2m)
                  bf(2)= i*q*pol1(2)*(-an2*exp2p+bn2*exp2m)
                  bf(3)= i*ag1      *( an2*exp2p+bn2*exp2m)
                  ef(1)= om*om*qol1(1)*eps_core*(an2*exp2p+bn2*exp2m)
                  ef(2)= om*om*qol1(2)*eps_core*(an2*exp2p+bn2*exp2m)
                  ef(3)= (0d0,0d0)
                else if (zz.gt.d/2d0) then                   ! medium 3
                  exp3=exp(-chi3*(zz-d/2d0))
                  bf(1)= an3*chi3*pol1(1)*exp3
                  bf(2)= an3*chi3*pol1(2)*exp3
                  bf(3)= an3*i*ag1       *exp3
                  ef(1)= om*om*qol1(1)*eps_clad3*an3*exp3
                  ef(2)= om*om*qol1(2)*eps_clad3*an3*exp3
                  ef(3)= (0d0,0d0)
                endif
              else if (jmode1.eq.1) then   ! TM-polarized basis state
                dn1=dnorm1(j1,jal1)
                dn2=dnorm2(j1,jal1)
                cn2=cnorm2(j1,jal1)
                cn3=cnorm3(j1,jal1)
                if (zz.lt.-d/2d0) then                       ! medium 1
                  exp1=exp(chi1*(zz+d/2d0))
                  bf(1)=dn1*qol1(1)*exp1
                  bf(2)=dn1*qol1(2)*exp1
                  bf(3)=(0d0,0d0)
                  ef(1)=-dn1*chi1*pol1(1)*exp1
                  ef(2)=-dn1*chi1*pol1(2)*exp1
                  ef(3)= dn1*i*ag1       *exp1
                else if (zz.ge.-d/2d0.and.zz.le.d/2d0) then  ! medium 2
                  exp2p=exp( i*q*zz)
                  exp2m=exp(-i*q*zz)
                  bf(1)=qol1(1)*(cn2*exp2p+dn2*exp2m)
                  bf(2)=qol1(2)*(cn2*exp2p+dn2*exp2m)
                  bf(3)=(0d0,0d0)
                  ef(1)= i*q*pol1(1)*(-cn2*exp2p+dn2*exp2m)
                  ef(2)= i*q*pol1(2)*(-cn2*exp2p+dn2*exp2m)
                  ef(3)= i*ag1      *( cn2*exp2p+dn2*exp2m)
                else if (zz.gt.d/2d0) then                   ! medium 3
                  exp3=exp(-chi3*(zz-d/2d0))
                  bf(1)=qol1(1)*cn3*exp3
                  bf(2)=qol1(2)*cn3*exp3
                  bf(3)=(0d0,0d0)
                  ef(1)= cn3*chi3*pol1(1)*exp3
                  ef(2)= cn3*chi3*pol1(2)*exp3
                  ef(3)= cn3*i*ag1       *exp3
                endif
              endif
c
c             adds contribution to field components
c
              do jcomp=1,3
                bfield(jcomp)=bfield(jcomp)+
     &            bf(jcomp)*expo*eigenv(jham1,jstate)/sqrt(ucarea)
                efield(jcomp)=efield(jcomp)+
     &            ef(jcomp)*expo*eigenv(jham1,jstate)*i/epsilon/
     &            (energ(jstate)/aret*2d0*pi/factor)/sqrt(ucarea)
              enddo
640         continue   ! ends sum over basis states
c
c           calculates modulus of fields
c
            emod2=0d0
            bmod2=0d0
            do jcomp=1,3
              emod2=emod2+efield(jcomp)*dconjg(efield(jcomp))
              bmod2=bmod2+bfield(jcomp)*dconjg(bfield(jcomp))
            enddo
c
c           Writes fields.
c           For each component, look whether real or imaginary part
c           is nonvanishing and uncomment relevant lines
c
            write (12,642) xx,yy,zz,eps_realspace
            if (jparxy.eq.0) then
              write (15,642) xx,yy,zz,dble(bfield(3))
              write (16,642) xx,yy,zz,dimag(bfield(3))
              write (17,642) xx,yy,zz,(efield(jcomp),jcomp=1,2)
              write (18,642) xx,yy,zz,
!     &         dble(efield(1)),dimag(efield(2)),dimag(bfield(3))
     &         dimag(efield(1)),dble(efield(2)),dble(bfield(3))
!              write (19,642) xx,yy,zz,dimag(bfield(3))
            else if (jparxy.eq.1) then
              write (15,642) xx,yy,zz,dble(efield(3))
              write (16,642) xx,yy,zz,dimag(efield(3))
              write (17,642) xx,yy,zz,(bfield(jcomp),jcomp=1,2)
              write (18,642) xx,yy,zz,dble(bfield(3))
!              write (19,642) xx,yy,zz,dimag(bfield(3))
            else if (jparxy.eq.-1) then
              write (15,642) xx,yy,zz,(efield(jcomp),jcomp=1,3)
              write (16,642) xx,yy,zz,(bfield(jcomp),jcomp=1,3)
            endif
            if (elnorm.ne.0) then
               write (13,643) xx,yy,zz,emod2/elnorm
            else 
               print *, 'elnorm=0!'
            endif
            write (14,643) xx,yy,zz,bmod2

632       continue
631       continue
          write (12,*)
          write (13,*)
          write (14,*)
          write (15,*)
          write (16,*)
          write (17,*)
          write (18,*)
630       continue

642       format(3(1x,f8.4),1x,3(1x,e9.3,' ',e9.3))
643       format(3(1x,f8.4),1x,2(1x,e10.4))
        endif       ! ends calculation of field amplitudes

c
c       ********************************************************
c
c        diffraction losses by second-order perturbation theory
c
c       ********************************************************
c
        if (jeigenv.eq.1) then
          do 500 jband=1,nbands
           freq2=energ2(jband)
           freq=sqrt(freq2)
           if (jband.ge.jlossmin.and.jband.le.jlossmax) then
            if (jparxy.eq.0.or.jparxy.eq.1) then
              call loss_sym (akx,aky,npw,freq2,jband,eigenv,jparxy,
     &                  nalpha_store,jmode_store,jham_store,freqim)
            else if (jparxy.eq.-1) then
              call loss_asym2 (akx,aky,npw,freq2,jband,eigenv,
     &                  nalpha_store,jmode_store,jham_store,freqim)
            endif
            if (freqim.lt.upper) freqim=0d0
            energim(jband)=freqim*aret/2d0/pi*factor
            if (freq.gt.0d0) qfacinv(jband)=2d0*freqim/freq
           else
            freqim=0d0
            energim(jband)=0d0
           endif
500       continue
        endif
c
c       shifts bands at Gamma point: 
c       sometimes it is necessary to get continuous curves
c
        if (abs(akxvec(jk)).lt.2*upper.and.
     &        abs(akyvec(jk)).lt.2*upper) then
          do jband=nbands,1+jshift,-1
            energ(jband)=energ(jband-jshift)
            energim(jband)=energim(jband-jshift)
            qfacinv(jband)=qfacinv(jband-jshift)
          enddo
          do jband=1,jshift
            energ(jband)=2d0*upper
            energim(jband)=0d0
            qfacinv(jband)=0d0
          enddo
        endif

        do jband=1,nbands
          bands    (jk,jband)=energ  (jband)
          bandsim  (jk,jband)=energim(jband)
          qinvbands(jk,jband)=qfacinv(jband)
        enddo

c
c       writes results
c

c
c       writes Im(omega) for dipole-active mode at k=0 (--> trends)
c
!        if (abs(akx).lt.2*upper.and.abs(aky).lt.2*upper) then
!          ncount=0
!          do jband=1,maxbands
!            diff1=energ(jband)-energ(jband+1)
!            diff2=energim(jband)-energim(jband+1)
!            if (abs(diff1).lt.1d-10.and.abs(diff2).lt.1d-10
!     &        .and.energim(jband).gt.1d-10) then
!              ncount=ncount+1
!              write (28,96) var,energ(jband),energim(jband)
!              if (ncount.eq.1) write (29,96) var
!     &                ,energ(jband)
!     &                ,energim(jband)
!!     &          ,(eps2_core-eps2_clad1),(eps2_core-eps2_clad1)**2
!!     &          ,fill
!            endif
!          enddo
!        endif
96      format(1x,f12.6,2x,e12.6,2(2x,e12.6))


c
c       ends loop over wavevectors
c
200   continue


c
c     Calculates group velocity "vgroup" (in units of "c"),
c     decay length "aldec" (in units of lattice constant "aret")
c     and propagation loss "aloss" (in units of decibel/aret).
c     
c     FOR W1 WAVEGUIDE ONLY, otherwise normalization factors
c     may depend on the direction of the Brillouin zone.
c     Calculation is done ONLY IF VERTICAL PARITY SYMMETRY IS IMPLEMENTED,
c     otherwise crossing between modes makes the group velocity curves 
c     difficult to disentagle: but it can be done, using care.
c
c     WARNING: the group velocity is defines IN ABSOLUTE VALUE,
c     otherwise expressions like max(vgroup,1d-20) do not work...
c
      if (jeigenv.eq.1.and.(jparkz.eq.0.or.jparkz.eq.1)) then
        do jk=1,nktot
          do jband=1,nbands
            if (jk.lt.nktot) then
              delkx=akxvec(jk+1)-akxvec(jk)
              delky=akyvec(jk+1)-akyvec(jk)
              delk=sqrt(delkx*delkx+delky*delky)
              vgroup(jk,jband)=
     &          abs(2d0*pi*(bands(jk+1,jband)-bands(jk,jband))/delk)
            else
              delkx=akxvec(jk)-akxvec(jk-1)
              delky=akyvec(jk)-akyvec(jk-1)
              delk=sqrt(delkx*delkx+delky*delky)
              vgroup(jk,jband)=
     &          abs(2d0*pi*(bands(jk,jband)-bands(jk-1,jband))/delk)
            endif
            akim=2d0*pi*bandsim(jk,jband)/max(vgroup(jk,jband),1d-20)
!            akim=2d0*pi*abs(bandsim(jk,jband)/
!     &            max(vgroup(jk,jband),1d-20) )
            aloss(jk,jband)=4.34*2d0*akim

            if (akim.gt.0) then
              aldec(jk,jband)=1d0/2d0/akim
            else
              aldec(jk,jband)=1d20
            endif
          enddo
        enddo
      endif

c
c     writes output for gnuplot
c
      write (6,23) nktot,nbands
23    format(1x,'number of wavevectors=',i3,/,' number of bands=',i4)
      write (2,24) jlattice,jbasis,jparxy,jparkz
      write (2,25) d,alength1,alength2,alength3,alength4
      write (2,26) eps1_clad1,eps2_clad1
      write (2,27) eps1_core ,eps2_core
      write (2,28) eps1_clad3,eps2_clad3
      write (2,29) npw,nalpha,factor
      write (21,24) jlattice,jbasis,jparkz
      write (21,25) d,alength1,alength2,alength3,alength4
      write (21,26) eps1_clad1,eps2_clad1
      write (21,27) eps1_core ,eps2_core
      write (21,28) eps1_clad3,eps2_clad3
      write (21,29) npw,nalpha,factor
      write (26,24) jlattice,jbasis,jparkz
      write (26,25) d,alength1,alength2,alength3,alength4
      write (26,26) eps1_clad1,eps2_clad1
      write (26,27) eps1_core ,eps2_core
      write (26,28) eps1_clad3,eps2_clad3
      write (26,29) npw,nalpha,factor
24    format('# jlattice=',i1,' jbasis=',i2,' jparxy=',i2,' jparkz=',i2)
25    format('# d=',f8.3,' alength1=',f7.3,
     &    ' alength2=',f7.3,' alength3=',f7.3,' alength4=',f7.3)
26    format('# clad1:  eps1=',f9.4,'  eps2=',f9.4)
27    format('# core :  eps1=',f9.4,'  eps2=',f9.4)
28    format('# clad3:  eps1=',f9.4,'  eps2=',f9.4)
29    format('# npw=',i3,'  nalpha=',i2,'  factor=',f12.6)


      do jband=1,nbands
!      do jband=1,maxbands
        write (2,31) jband
        write (21,31) jband
31      format('#      band number ',i3)
        do jk=1,nktot
         if (bands(jk,jband).gt.upper) write (2,180) 
     &         ax(jk),bands(jk,jband),bandsim(jk,jband)
     &         ,vgroup(jk,jband)
     &         ,aloss(jk,jband)
!     &         ,aldec(jk,jband)
!     &         ,1d0/max(qinvbands(jk,jband),1d-20)
         if (bands(jk,jband).gt.upper) write (21,180) 
     &         ax(jk),bands(jk,jband),bandsim(jk,jband)
        enddo
        write (2,*)
        write (21,*)

        if (jband.ge.jlossmin.and.jband.le.jlossmax) then
          write (26,31) jband
          do jk=1,nktot
             write (26,180) 
     &         ax(jk),bands(jk,jband)
     &         ,bandsim(jk,jband)
     &         ,abs(vgroup(jk,jband))
     &         ,aloss(jk,jband)
!     &         ,aldec(jk,jband)
!     &         ,1d0/max(qinvbands(jk,jband),1d-20)
          enddo
          write (26,*)
        endif

      enddo
      do jk=1,nktot                 ! light line for core
        write (9,180) ax(jk),alcore(jk)
      enddo
      write (9,*)
      do jk=1,nktot                 ! light line for upper cladding
        write (9,180) ax(jk),alclad1(jk)
      enddo
      write (9,*)
      do jk=1,nktot                 ! light line for lower cladding
        write (9,180) ax(jk),alclad3(jk)
      enddo
      write (9,*)
!      do jk=1,nktot                 ! light line for air
!        write (9,180) ax(jk),alair(jk)
!      enddo
!      write (9,*)
!180   format (1x,f12.6,2x,5(2x,e12.6))
180   format (1x,f12.6,2x,5(1x,e14.8))
181   format (1x,f10.6,2x,f10.6,2x,f10.6,4x,e12.6,4x,f12.6)
      write (*,182) (ax(jk),jk=1,nktot,nk)
182   format (' set xtics (',6(f8.5,','),f8.5,')')
      do jk=nk+1,nktot-2,nk
        write (7,180) ax(jk),0d0
        write (7,180) ax(jk),1d6
        write (7,180) 
      enddo

c
c     Writes light lines in all Brillouin zones:
c     sometimes useful to understand diffraction processes 
c     with nonzero reciprocal lattice vectors
c
!      do j1=1,maxbands
!        do jk=1,nktot
!          write(33,189) ax(jk),alightstore(jk,j1)
!        enddo
!        write (33,*)
!      enddo
188   format(1x,2(f12.6,2x),2x,e12.6)
189   format(1x,2f12.6,2x,2x,e12.6)


9995  continue    ! ends loop over variable VAR

      stop
      end
c
      double precision function fwaveguide (omega)
c
c     function whose zeros give waveguide modes
c
      implicit none
      integer jparxy
      real*8 omega,q,chi1,chi3,ag,eps_core,eps_clad1,eps_clad3
      real*8 aacos,aasin,d,funte,funtm
      common/jparxy/jparxy
      common/paramf0/ag,d,eps_core,eps_clad1,eps_clad3
      q    =sqrt(eps_core*omega*omega-ag*ag)
      chi1 =sqrt(ag*ag-eps_clad1*omega*omega)
      chi3 =sqrt(ag*ag-eps_clad3*omega*omega)
      if (jparxy.eq.-1) then
        aacos=cos(q*d)
        aasin=sin(q*d)
        funte=q*(chi1+chi3)*aacos+(chi1*chi3-q*q)*aasin
        funtm=q/eps_core*(chi1/eps_clad1+chi3/eps_clad3)*aacos+
     &     (chi1*chi3/eps_clad1/eps_clad3-q*q/eps_core/eps_core)*aasin
      else if (jparxy.eq.0) then
        aacos=cos(q*d/2d0)
        aasin=sin(q*d/2d0)
        funte=q*aasin         -chi1*aacos
        funtm=q*aacos/eps_core+chi1*aasin/eps_clad1
      else if (jparxy.eq.1) then
        aacos=cos(q*d/2d0)
        aasin=sin(q*d/2d0)
        funte=q*aacos         +chi1*aasin
        funtm=q*aasin/eps_core-chi1*aacos/eps_clad1
      endif
      fwaveguide=funte*funtm
      return
      end
c
      subroutine decouwga (akx,aky,npw,h,jlattice,jparkz,dir,nham,
     &                     nalpha_store,jmode_store,jham_store)
c
c     Decouples hamiltonian matrix in blocks of even and odd parities
c     w.r.t. a specified mirror plane, for waveguide code.
c
      implicit none
      integer jlattice,jpol,jparkz,maxpw,maxham,npw,maxg,j1,j2,j3
      integer neven,nodd,nham,jeven,jodd,jal1,jal2,jham1,jham2,jham3
      integer maxalpha
      parameter (maxpw=1100,maxham=2200,maxg=2e5,maxalpha=8)
      integer nalpha_store(maxpw),jmode_store(maxpw,maxalpha)
      integer jsym(maxpw),jham_store(maxpw,maxalpha)
      integer jmode1,jmode2,nalpha1,nalpha2,jwork(maxham)
      real*8 h(maxham,maxham),work(maxham,maxham),u(maxham,maxham)
      real*8 akx,aky,gxvec(maxg),gyvec(maxg),upper,gx1sym,gy1sym,gx2,gy2
      real*8 cos45,pi,gx1,gy1
      character*4 dir
      parameter (upper=1d-10)
      common/gvec/gxvec,gyvec
      common/matrix/u
      common/iarray/jwork
      pi=4d0*atan(1d0)
c
c     for each reciprocal lattice vector, finds the symmetric one
c
      do 10 j1=1,npw
        gx1=gxvec(j1)
        gy1=gyvec(j1)
        if (jlattice.eq.1) then
          if (dir(1:4).eq.'xgam') then
            gx1sym= gx1
            gy1sym=-gy1
          else if (dir(1:4).eq.'ygam') then
            gx1sym=-gx1
            gy1sym= gy1
          else if (dir(1:4).eq.'gamm') then
            gx1sym=gy1
            gy1sym=gx1
          else
            stop ' from DECOUWGA: wrong value of string DIR'
          endif
        else if (jlattice.eq.2) then
          if (dir(1:4).eq.'mgam') then
            gx1sym=-gx1
            gy1sym= gy1
          else if (dir(1:4).eq.'gamk') then
            gx1sym=(-gx1+sqrt(3d0)*gy1)/2d0
            gy1sym=(sqrt(3d0)*gx1+gy1)/2d0
!            gx1sym= gx1   ! depends on choice of point K in triang_points
!            gy1sym=-gy1
          else
            stop ' from DECOUWGA: wrong value of string DIR'
          endif
        else if (jlattice.eq.4) then
          if (dir(1:4).eq.'xgam') then
            gx1sym= gx1
            gy1sym=-gy1
          else
            stop ' from DECOUWGA: wrong value of string DIR'
          endif
        else
          stop ' DECOUWGA implemented only for jlattice=1,2,4'
        endif
        jsym(j1)=0
        do 20 j2=1,npw
          gx2=gxvec(j2)
          gy2=gyvec(j2)
          if (abs(gx2-gx1sym).le.upper.and.abs(gy2-gy1sym).lt.upper) 
     &      jsym(j1)=j2
20      continue
        if (jsym(j1).eq.0) stop ' from DECOUWGA: no sym. wavevector'
!        write (99,999) j1,gxvec(j1),gyvec(j1),
!     &                 jsym(j1),gxvec(jsym(j1)),gyvec(jsym(j1))
10    continue
!      write (99,*)
999   format (1x,3(i3,2x,f12.6,2x,f12.6,2x,2x))
c
c     calculates the unitary matrix which decouples the hamiltonian
c     in the basis of even and odd states
c
      cos45=sqrt(0.5d0)
      do 30 j1=1,npw
      do 30 jal1=1,nalpha_store(j1)
        do 30 j2=1,npw
        do 30 jal2=1,nalpha_store(j2)
          jham1=jham_store(j1,jal1)
          jham2=jham_store(j2,jal2)
          u(jham1,jham2)=0d0
30    continue
      do 35 j1=1,npw
        j2=jsym(j1)
        nalpha1=nalpha_store(j1)
        nalpha2=nalpha_store(j2)
        if (nalpha1.ne.nalpha2) stop 'from DECOUWGA: nalpha1.ne.nalpha2'
        do 36 jal1=1,nalpha1
          jham1=jham_store(j1,jal1)
          if (j2.eq.j1) then
            u(jham1,jham1)=1d0
          else if (j2.gt.j1) then
            jham2=jham_store(j2,jal1)
            jmode1=jmode_store(j1,jal1)
            jmode2=jmode_store(j2,jal1)
            if (jmode1.ne.jmode2) stop 'from DECOUWGA: jmode1.ne.jmode2'
            u(jham1,jham1)=cos45
            u(jham1,jham2)=cos45
            u(jham2,jham1)=-cos45
            u(jham2,jham2)=cos45
          endif
36      continue
35    continue
c
c     counts number of even and odd states:
c       for TE modes (B in plane of incidence) even states are spatially odd
c       for TM modes (E in plane of incidence) even states are spatially even
c
      neven=0
      nodd=0
      do 38 j1=1,npw
        nalpha1=nalpha_store(j1)
        j2=jsym(j1)
        do 39 jal1=1,nalpha1
          jmode1=jmode_store(j1,jal1)
          if (j2.ge.j1) then
            if (jmode1.eq.0) nodd=nodd+1
            if (jmode1.eq.1) neven=neven+1
          else 
            if (jmode1.eq.0) neven=neven+1
            if (jmode1.eq.1) nodd =nodd+1
          endif
39      continue
38    continue
      if (neven+nodd.ne.nham) stop ' from DECOUPLE: neven+nodd.ne.nham'
!      call wrmat (u,maxham,nham,1d0,0d0)
c
c     transforms hamiltonian in the basis of even/odd states
c
      do 40 j1=1,npw
      do 40 jal1=1,nalpha_store(j1)
        jham1=jham_store(j1,jal1)
        do 40 j2=1,npw
        do 40 jal2=1,nalpha_store(j2)
          jham2=jham_store(j2,jal2)
!          work(jham1,jham2)=0d0
!          do 45 jham3=1,nham
!           work(jham1,jham2)=work(jham1,jham2)
!     &                      +u(jham1,jham3)*h(jham3,jham2)
!45        continue
          j3=jsym(j1)
          jham3=jham_store(j3,jal1)
          if (j3.eq.j1) then
            work(jham1,jham2)=h(jham1,jham2)
          else
            work(jham1,jham2)=u(jham1,jham1)*h(jham1,jham2)
     &                       +u(jham1,jham3)*h(jham3,jham2)
          endif
40    continue
      do 50 j1=1,npw
      do 50 jal1=1,nalpha_store(j1)
        jham1=jham_store(j1,jal1)
        do 50 j2=1,npw
        do 50 jal2=1,nalpha_store(j2)
          jham2=jham_store(j2,jal2)
!        h(jham1,jham2)=0d0
!        do 55 jham3=1,nham
!          h(jham1,jham2)=h(jham1,jham2)+work(jham1,jham3)*u(jham2,jham3)
!55      continue
        j3=jsym(j2)
        jham3=jham_store(j3,jal2)
        if (j3.eq.j2) then
           h(jham1,jham2)=work(jham1,jham2)
        else
           h(jham1,jham2)=work(jham1,jham2)*u(jham2,jham2)
     &                   +work(jham1,jham3)*u(jham2,jham3)
        endif
50    continue
!      call repulisti (h,maxham,nham)
!      call wrmat (h,maxham,nham,1d0,0d0)
c
c     reorders blocks in order that even or odd states come first
c
      jeven=0
      jodd=0
      do 60 j1=1,npw
        j2=jsym(j1)
        do 60 jal1=1,nalpha_store(j1)
        jmode1=jmode_store(j1,jal1)
        jham1=jham_store(j1,jal1)
        if ((j2.ge.j1.and.jmode1.eq.0).or.
     &      (j2.lt.j1.and.jmode1.eq.1)) then ! index j1 belongs to odd sector
          jodd=jodd+1
          if (jparkz.eq.0) jwork(jham1)=neven+jodd
          if (jparkz.eq.1) jwork(jham1)=jodd
        else if ((j2.ge.j1.and.jmode1.eq.1).or.
     &           (j2.lt.j1.and.jmode1.eq.0)) then ! j1 belongs to even sector
          jeven=jeven+1
          if (jparkz.eq.0) jwork(jham1)=jeven
          if (jparkz.eq.1) jwork(jham1)=nodd+jeven
        endif
60    continue
      do 62 jham1=1,nham
      do 62 jham2=1,nham
        work(jwork(jham1),jwork(jham2))=h(jham1,jham2)
62    continue
      do 65 jham1=1,nham
      do 65 jham2=1,nham
        h(jham1,jham2)=work(jham1,jham2)
65    continue
!      call wrmat (h,maxham,nham,1d0,0d0)
!      write (6,*) 'neven=',neven,'  nodd=',nodd
      if (jparkz.eq.0) nham=neven
      if (jparkz.eq.1) nham=nodd
      return
      end
c
      subroutine restore (eigenv,nham,nham_store)
c
c     restores eigenvectors in basis of plane waves
c
      implicit none
      integer maxham,nham,nham_store,jham1,jham2,jham3
      parameter (maxham=2200)
      real*8 eigenv(maxham,maxham)
      real*8 u(maxham,maxham),work(maxham,maxham),sum
      integer jwork(maxham)
      common/matrix/u
      common/iarray/jwork
c
c     defines eigenvector component to be zero for all rows beyod nham
c
      do jham2=1,nham
        do jham1=nham+1,nham_store
          eigenv(jham1,jham2)=0d0
        enddo
      enddo
c
c     restores eigenvector in basis of even/odd combinations of PWs
c     (i.e., before using index jwork)
c
      do jham2=1,nham
        do jham1=1,nham_store
          work(jham1,jham2)=eigenv(jwork(jham1),jham2)
        enddo
      enddo
c
c     performs unitary transformation with inverse of matrix u
c
      do jham2=1,nham
        do jham1=1,nham_store
          sum=0d0
          do jham3=1,nham_store
            sum=sum+work(jham3,jham2)*u(jham3,jham1)
          enddo
          eigenv(jham1,jham2)=sum
        enddo
      enddo
      return
      end
c
c     ****************************************************************
c
      subroutine loss_sym (akx,aky,npw,freq2,jband,eigenv,jparxy,
     &                   nalpha_store,jmode_store,jham_store,freqim)
c
c     ****************************************************************
c
c     calculates diffraction losses (imaginary part of frequency)
c     by time-dependent perturbation theory, for the case
c     of a symmetric waveguide
c
      implicit none
      integer maxpw,maxham,npw,maxg,maxalpha,jband,jparxy
      parameter (maxpw=1100,maxham=2200,maxg=2e5,maxalpha=8)
      integer nalpha_store(maxpw),jmode_store(maxpw,maxalpha)
      integer jsym(maxpw),jham_store(maxpw,maxalpha)
      integer jsign,j1,j2,jal1,jham1,jmode1
      real*8 akx,aky,freq2,freq,freqim,freq2im,spher0,pi,add
      real*8 ag1,d,eps_core,eps_clad1,eps_clad3,akmod,dos,alight
      real*8 gxvec(maxg),gyvec(maxg),eigenv(maxham,maxham)
      real*8 akz1,akz2,akz3,aq1,aq2,aq3,upper,diff,scal,agx2,agy2
      real*8 checkmax,check1,check2,check3,check4,check5
      real*8 pol1(2),pol2(2),qol1(2),qol2(2)
      real*8 omega1,agx1,agy1,agmod,q,chi1,chi3,aiminus,aiplus
      real*8 ft1inv(maxpw,maxpw),ft2inv(maxpw,maxpw),ft3inv(maxpw,maxpw)
      real*8 om_store(maxpw,maxalpha),q_store(maxpw,maxalpha)
      real*8 chi1_store(maxpw,maxalpha),chi3_store(maxpw,maxalpha)

      complex*16 i,elemte,elemtm,couplte,coupltm,t11,t12,t21,t22
      complex*16 ainttete,ainttetm,ainttmte,ainttmtm
      complex*16 checktete,checktetm,checktmte,checktmtm
      complex*16 wnorm1,xnorm1,ynorm1,znorm1
      complex*16 wnorm2,xnorm2,ynorm2,znorm2
      complex*16 wnorm3,xnorm3,ynorm3,znorm3
      complex*16 bnorm1(maxpw,maxalpha),bnorm2(maxpw,maxalpha)
      complex*16 anorm2(maxpw,maxalpha),anorm3(maxpw,maxalpha)
      complex*16 dnorm1(maxpw,maxalpha),dnorm2(maxpw,maxalpha)
      complex*16 cnorm2(maxpw,maxalpha),cnorm3(maxpw,maxalpha)
      parameter (i=(0d0,1d0),upper=1d-12)

      common/paramf0/ag1,d,eps_core,eps_clad1,eps_clad3
      common/gvec/gxvec,gyvec
      common/store/om_store,q_store,chi1_store,chi3_store
      common/an/anorm2,anorm3,bnorm1,bnorm2,cnorm2,cnorm3,dnorm1,dnorm2
      common/ftinv/ft1inv,ft2inv,ft3inv

      pi=4d0*atan(1d0)
      freq=sqrt(freq2)

      freq2im=0d0
c
c     starts loop over reciprocal lattice vectors or Brillouin zones
c     for leaky modes of effective waveguide
c
      do 200 j2=1,npw
       agx2=akx+gxvec(j2)
       agy2=aky+gyvec(j2)
       akmod=sqrt(agx2*agx2+agy2*agy2)
       alight=akmod/sqrt(eps_clad1)
       if (freq.le.alight) goto 200 ! mode is below light line --> no contribution to losses
c
c      radiation modes of effective waveguide
c
       akz1=sqrt(eps_clad1*freq2-akmod*akmod)
       akz2=sqrt(eps_core *freq2-akmod*akmod)
       akz3=sqrt(eps_clad3*freq2-akmod*akmod)
c
c      TE modes
c
       aq1=akz1
       aq2=akz2
       aq3=akz3
       t11=((aq3+aq2)*(aq2+aq1)*exp( i*akz2*d)
     &     +(aq3-aq2)*(aq2-aq1)*exp(-i*akz2*d))/4d0/aq3/aq2
       t22=((aq3+aq2)*(aq2+aq1)*exp(-i*akz2*d)
     &     +(aq3-aq2)*(aq2-aq1)*exp( i*akz2*d))/4d0/aq3/aq2
       t12=((aq3+aq2)*(aq2-aq1)*exp( i*akz2*d)
     &     +(aq3-aq2)*(aq2+aq1)*exp(-i*akz2*d))/4d0/aq3/aq2
       t21=((aq3-aq2)*(aq2+aq1)*exp( i*akz2*d)
     &     +(aq3+aq2)*(aq2-aq1)*exp(-i*akz2*d))/4d0/aq3/aq2

c
c      normalization is imposed for outgoing field components
c
       xnorm1=1d0/sqrt(2d0*eps_clad1)
       if (jparxy.eq.0) then
         wnorm1=(1d0-t12)/t11*xnorm1 
       else if (jparxy.eq.1) then
         wnorm1=-(1d0+t12)/t11*xnorm1
       endif

       wnorm2=((aq2+aq1)*wnorm1+(aq2-aq1)*xnorm1)
     &        *exp( i*akz2*d/2d0)/2d0/aq2
       xnorm2=((aq2-aq1)*wnorm1+(aq2+aq1)*xnorm1)
     &        *exp(-i*akz2*d/2d0)/2d0/aq2
       wnorm3=t11*wnorm1+t12*xnorm1
       xnorm3=t21*wnorm1+t22*xnorm1
!       write (*,12) wnorm1,xnorm1,wnorm2,xnorm2,wnorm3,xnorm3
       jsign=1-2*jparxy
       check1=abs(xnorm1-jsign*t11/(1d0-jsign*t12)*wnorm1)
       check2=abs(wnorm2-jsign*xnorm2)
       check3=abs(xnorm3-jsign*wnorm1)
       check4=abs(wnorm3-jsign*xnorm1)
       check5=abs(wnorm3)-abs(wnorm1)
       checkmax=max(check1,check2,check3,check4,check5)
       if (checkmax.gt.upper) then
         print *, 'error in subroutine LOSS_SYM while calculating TE'
         print *, 'checkmax too large'
         print *, checkmax
       endif
12     format(1x,6(e9.3,',',e9.3,1x))
c
c      TM modes
c
       aq1=akz1/eps_clad1
       aq2=akz2/eps_core
       aq3=akz3/eps_clad3
       t11=((aq3+aq2)*(aq2+aq1)*exp( i*akz2*d)
     &     +(aq3-aq2)*(aq2-aq1)*exp(-i*akz2*d))/4d0/aq3/aq2
       t22=((aq3+aq2)*(aq2+aq1)*exp(-i*akz2*d)
     &     +(aq3-aq2)*(aq2-aq1)*exp( i*akz2*d))/4d0/aq3/aq2
       t12=((aq3+aq2)*(aq2-aq1)*exp( i*akz2*d)
     &     +(aq3-aq2)*(aq2+aq1)*exp(-i*akz2*d))/4d0/aq3/aq2
       t21=((aq3-aq2)*(aq2+aq1)*exp( i*akz2*d)
     &     +(aq3+aq2)*(aq2-aq1)*exp(-i*akz2*d))/4d0/aq3/aq2

c
c      normalization is imposed for outgoing field components
c
       znorm1=1d0/sqrt(2d0)
       if (jparxy.eq.0) then
         ynorm1=-(1d0+t12)/t11*znorm1
       else if (jparxy.eq.1) then
         ynorm1= (1d0-t12)/t11*znorm1
       endif

       ynorm2=((aq2+aq1)*ynorm1+(aq2-aq1)*znorm1)
     &        *exp( i*akz2*d/2d0)/2d0/aq2
       znorm2=((aq2-aq1)*ynorm1+(aq2+aq1)*znorm1)
     &        *exp(-i*akz2*d/2d0)/2d0/aq2
       ynorm3=t11*ynorm1+t12*znorm1
       znorm3=t21*ynorm1+t22*znorm1
!       write (*,512) ynorm1,znorm1,ynorm2,znorm2,ynorm3,znorm3
       check1=abs(znorm1+jsign*t11/(1d0+jsign*t12)*ynorm1)
       check2=abs(ynorm2+jsign*znorm2)
       check3=abs(znorm3+jsign*ynorm1)
       check4=abs(ynorm3+jsign*znorm1)
       check5=abs(ynorm3)-abs(ynorm1)
       checkmax=max(check1,check2,check3,check4,check5)
       if (checkmax.gt.upper) then
         print *, 'error in subroutine LOSS_SYM while calculating TM'
         print *, 'checkmax too large'
         print *, checkmax
       endif

       pol2(1)= agx2/akmod
       pol2(2)= agy2/akmod
       qol2(1)=-agy2/akmod
       qol2(2)= agx2/akmod

c
c      matrix element with guided mode: sum over basis states
c 
       elemte=dcmplx(0d0,0d0)
       elemtm=dcmplx(0d0,0d0)

       do 100 j1=1,npw
       do 100 jal1=1,nalpha_store(j1)
        jham1=jham_store(j1,jal1)
        jmode1=jmode_store(j1,jal1)
        omega1=om_store(j1,jal1)
        agx1=akx+gxvec(j1)
        agy1=aky+gyvec(j1)
        agmod=sqrt(agx1*agx1+agy1*agy1)
        q=q_store(j1,jal1) 
        chi1=chi1_store(j1,jal1)
        chi3=chi3_store(j1,jal1)
        pol1(1)= agx1/agmod
        pol1(2)= agy1/agmod
        qol1(1)=-agy1/agmod
        qol1(2)= agx1/agmod

        aiminus=d*spher0((q-akz2)*d/2d0)
        aiplus =d*spher0((q+akz2)*d/2d0)

        if (jmode1.eq.0) then ! TE polarized basis state

          ainttete=omega1*omega1*freq*scal(qol1,qol2,2) * (
     &    eps_clad1*eps_clad1*ft1inv(j1,j2)*dconjg(bnorm1(j1,jal1))
     &    *(wnorm1/(chi1+i*akz1)+xnorm1/(chi1-i*akz1))
     &   +eps_clad3*eps_clad3*ft3inv(j1,j2)*dconjg(anorm3(j1,jal1))
     &    *(wnorm3/(chi3-i*akz3)+xnorm3/(chi3+i*akz3))
     &   +eps_core*eps_core*ft2inv(j1,j2)*(
     &    (dconjg(anorm2(j1,jal1))*wnorm2
     &    +dconjg(bnorm2(j1,jal1))*xnorm2)*aiminus
     &   +(dconjg(anorm2(j1,jal1))*xnorm2
     &    +dconjg(bnorm2(j1,jal1))*wnorm2)*aiplus ) 
     &     )

          ainttetm=i*omega1*omega1*scal(qol1,pol2,2)* (
     &    eps_clad1*ft1inv(j1,j2)*dconjg(bnorm1(j1,jal1))
     &    *akz1*(-ynorm1/(chi1+i*akz1)+znorm1/(chi1-i*akz1))
     &   +eps_clad3*ft3inv(j1,j2)*dconjg(anorm3(j1,jal1))
     &    *akz3*(-ynorm3/(chi3-i*akz3)+znorm3/(chi3+i*akz3))
     &   +eps_core*ft2inv(j1,j2)*akz2* (
     &    (-dconjg(anorm2(j1,jal1))*ynorm2
     &     +dconjg(bnorm2(j1,jal1))*znorm2)*aiminus
     &   +( dconjg(anorm2(j1,jal1))*znorm2
     &     -dconjg(bnorm2(j1,jal1))*ynorm2)*aiplus ) 
     &      )

          couplte=ainttete
          coupltm=ainttetm

          checktete=omega1*omega1*freq*scal(qol1,qol2,2) * (
     &    eps_clad3*eps_clad3*ft3inv(j1,j2)*dconjg(anorm3(j1,jal1))
     &    *(wnorm3/(chi3-i*akz3)+xnorm3/(chi3+i*akz3))*2d0
     &   +eps_core*eps_core*ft2inv(j1,j2)*(
     &    (dconjg(anorm2(j1,jal1))*wnorm2
     &    +dconjg(bnorm2(j1,jal1))*xnorm2)*aiminus
     &   +(dconjg(anorm2(j1,jal1))*xnorm2
     &    +dconjg(bnorm2(j1,jal1))*wnorm2)*aiplus ) )

          diff=abs(ainttete-checktete)
          if (diff.gt.1d-6) print *, diff

          checktetm=i*omega1*omega1*scal(qol1,pol2,2)* (
     &    eps_clad3*ft3inv(j1,j2)*dconjg(anorm3(j1,jal1))
     &    *akz3*(-ynorm3/(chi3-i*akz3)+znorm3/(chi3+i*akz3))*2d0
     &   +eps_core*ft2inv(j1,j2)*akz2* (
     &    (-dconjg(anorm2(j1,jal1))*ynorm2
     &     +dconjg(bnorm2(j1,jal1))*znorm2)*aiminus
     &   +( dconjg(anorm2(j1,jal1))*znorm2
     &     -dconjg(bnorm2(j1,jal1))*ynorm2)*aiplus ) )

          diff=abs(ainttetm-checktetm)
          if (diff.gt.1d-6) print *, diff

        else if (jmode1.eq.1)  then ! TM polarized basis state

          ainttmte=freq*scal(pol1,qol2,2)* (
     &    -eps_clad1*ft1inv(j1,j2)*chi1*dconjg(dnorm1(j1,jal1))
     &    *(wnorm1/(chi1+i*akz1)+xnorm1/(chi1-i*akz1))
     &    +eps_clad3*ft3inv(j1,j2)*chi3*dconjg(cnorm3(j1,jal1))
     &    *(wnorm3/(chi3-i*akz3)+xnorm3/(chi3+i*akz3))
     &    -i*eps_core*ft2inv(j1,j2)*q* (
     &    (-dconjg(cnorm2(j1,jal1))*wnorm2
     &     +dconjg(dnorm2(j1,jal1))*xnorm2)*aiminus
     &    +(dconjg(dnorm2(j1,jal1))*wnorm2
     &     -dconjg(cnorm2(j1,jal1))*xnorm2)*aiplus ) 
     &      )


          ainttmtm=
     &    +ft1inv(j1,j2)*dconjg(dnorm1(j1,jal1)) * (
     &      (i*chi1*akz1*scal(pol1,pol2,2)+agmod*akmod)
     &      *ynorm1/(chi1+i*akz1)
     &     +(-i*chi1*akz1*scal(pol1,pol2,2)+agmod*akmod)
     &      *znorm1/(chi1-i*akz1)                  )
     &    +ft3inv(j1,j2)*dconjg(cnorm3(j1,jal1)) * (
     &      (-i*chi3*akz3*scal(pol1,pol2,2)+agmod*akmod)
     &      *ynorm3/(chi3-i*akz3)
     &     +( i*chi3*akz3*scal(pol1,pol2,2)+agmod*akmod)
     &      *znorm3/(chi3+i*akz3)                  )
     &    +ft2inv(j1,j2)*(
     &     (dconjg(cnorm2(j1,jal1))*ynorm2
     &     +dconjg(dnorm2(j1,jal1))*znorm2)
     &    *( q*akz2*scal(pol1,pol2,2)+agmod*akmod)*aiminus
     &    +(dconjg(cnorm2(j1,jal1))*znorm2
     &     +dconjg(dnorm2(j1,jal1))*ynorm2)
     &    *(-q*akz2*scal(pol1,pol2,2)+agmod*akmod)*aiplus)

          couplte=ainttmte
          coupltm=ainttmtm

          checktmte=freq*scal(pol1,qol2,2)* (
     &    eps_clad3*ft3inv(j1,j2)*chi3*dconjg(cnorm3(j1,jal1))
     &    *(wnorm3/(chi3-i*akz3)+xnorm3/(chi3+i*akz3))*2d0
     &    -i*eps_core*ft2inv(j1,j2)*q* (
     &    (-dconjg(cnorm2(j1,jal1))*wnorm2
     &     +dconjg(dnorm2(j1,jal1))*xnorm2)*aiminus
     &    +(dconjg(dnorm2(j1,jal1))*wnorm2
     &     -dconjg(cnorm2(j1,jal1))*xnorm2)*aiplus ) )

          diff=abs(ainttmte-checktmte)
          if (diff.gt.1d-6) print *, diff

          checktmtm=
     &     ft3inv(j1,j2)*dconjg(cnorm3(j1,jal1)) * 2d0* (
     &      (-i*chi3*akz3*scal(pol1,pol2,2)+agmod*akmod)
     &      *ynorm3/(chi3-i*akz3)
     &     +( i*chi3*akz3*scal(pol1,pol2,2)+agmod*akmod)
     &      *znorm3/(chi3+i*akz3)                  )
     &    +ft2inv(j1,j2)*(
     &     (dconjg(cnorm2(j1,jal1))*ynorm2
     &     +dconjg(dnorm2(j1,jal1))*znorm2)
     &    *( q*akz2*scal(pol1,pol2,2)+agmod*akmod)*aiminus
     &    +(dconjg(cnorm2(j1,jal1))*znorm2
     &     +dconjg(dnorm2(j1,jal1))*ynorm2)
     &       *(-q*akz2*scal(pol1,pol2,2)+agmod*akmod)*aiplus)

          diff=abs(ainttmtm-checktmtm)
          if (diff.gt.1d-6) print *, diff

        endif


c
c       adds contribution from basis state (j1,jal1)-->jham1
c

        elemte=elemte+couplte*eigenv(jham1,jband)
        elemtm=elemtm+coupltm*eigenv(jham1,jband)

100    continue

c
c      density of states
c
       akz1=sqrt(eps_clad1*freq2-akmod*akmod)
       dos=eps_clad1/4d0/pi/akz1
c
c      imaginary part of squared frequency
c
       add=pi*(elemte*dconjg(elemte)+elemtm*dconjg(elemtm))*dos
       freq2im=freq2im+add
200   continue           ! ends loop over rec. lattice vector or BZ
c
c     imaginary part of frequency --> square root
c
      freqim=dimag(sqrt(freq2+i*freq2im))
c
c     ends calculation of diffraction losses for given wavevector
c
      return
      end
c
c     ****************************************************************
c
      subroutine loss_asym2 (akx,aky,npw,freq2,jband,eigenv,
     &               nalpha_store,jmode_store,jham_store,freqim)
c
c     ****************************************************************
c
c     calculates diffraction losses (imaginary part of frequency)
c     by time-dependent perturbation theory, for the general case
c     of an asymmetric waveguide
c
c     upgrade 17/01/03: considers case when frequency is between
c     two light lines. akz1 or akz3 can be complex
c
      implicit none
      integer maxpw,maxham,npw,maxg,maxalpha,jband,jlight1,jlight3
      parameter (maxpw=1100,maxham=2200,maxg=2e5,maxalpha=8)
      integer nalpha_store(maxpw),jmode_store(maxpw,maxalpha)
      integer jsym(maxpw),jham_store(maxpw,maxalpha)
      integer jsign,j1,j2,jal1,jham1,jmode1,j
      real*8 akx,aky,freq2,freq,freqim,freq2im,spher0,pi
      real*8 ag1,d,eps_core,eps_clad1,eps_clad3,akmod,dos1,dos3
      real*8 gxvec(maxg),gyvec(maxg),eigenv(maxham,maxham)
      real*8 akz2,aq2,upper,scal,agx2,agy2,add
      real*8 checkmax,check1,check2,check3,check4,check5,check6
      real*8 pol1(2),pol2(2),qol1(2),qol2(2)
      real*8 omega1,agx1,agy1,agmod,q,chi1,chi3,aiminus,aiplus
      real*8 ft1inv(maxpw,maxpw),ft2inv(maxpw,maxpw),ft3inv(maxpw,maxpw)
      real*8 om_store(maxpw,maxalpha),q_store(maxpw,maxalpha)
      real*8 chi1_store(maxpw,maxalpha),chi3_store(maxpw,maxalpha)
      real*8 znorm(2),alight1,alight3

      complex*16 akz1,akz3,aq1,aq3
      complex*16 i,elemte(2),elemtm(2),couplte(2),coupltm(2),det
      complex*16 t11,t12,t21,t22,t11half,t12half,t21half,t22half
      complex*16 ainttete(2),ainttetm(2),ainttmte(2),ainttmtm(2)
      complex*16 wn1(2),xn1(2),yn1(2),zn1(2)
      complex*16 wn2(2),xn2(2),yn2(2),zn2(2)
      complex*16 wn3(2),xn3(2),yn3(2),zn3(2)
      complex*16 wn1c(2),xn1c(2),yn1c(2),zn1c(2)
      complex*16 wn2c(2),xn2c(2),yn2c(2),zn2c(2)
      complex*16 wn3c(2),xn3c(2),yn3c(2),zn3c(2)
      complex*16 bnorm1(maxpw,maxalpha),bnorm2(maxpw,maxalpha)
      complex*16 anorm2(maxpw,maxalpha),anorm3(maxpw,maxalpha)
      complex*16 dnorm1(maxpw,maxalpha),dnorm2(maxpw,maxalpha)
      complex*16 cnorm2(maxpw,maxalpha),cnorm3(maxpw,maxalpha)
      complex*16 sll,srr,slr
      parameter (i=(0d0,1d0),upper=1d-12)

      common/paramf0/ag1,d,eps_core,eps_clad1,eps_clad3
      common/gvec/gxvec,gyvec
      common/store/om_store,q_store,chi1_store,chi3_store
      common/an/anorm2,anorm3,bnorm1,bnorm2,cnorm2,cnorm3,dnorm1,dnorm2
      common/ftinv/ft1inv,ft2inv,ft3inv

      pi=4d0*atan(1d0)
      freq=sqrt(freq2)

      freq2im=0d0
c
c     starts loop over reciprocal lattice vectors or Brillouin zones
c     for leaky modes of effective waveguide
c
      do 200 j2=1,npw
       agx2=akx+gxvec(j2)
       agy2=aky+gyvec(j2)
       akmod=sqrt(agx2*agx2+agy2*agy2)
       alight1=akmod/sqrt(eps_clad1)
       alight3=akmod/sqrt(eps_clad3)

       if (freq.le.min(alight1,alight3)) goto 200 ! no losses below light lines
       if (freq.gt.alight1) then
         jlight1=1
       else if (freq.eq.alight1) then
         print *, 'from LOSS_ASYM2: freq=alight1'
         goto 200
       else 
         jlight1=0
       endif
       if (freq.gt.alight3) then
         jlight3=1
       else if (freq.eq.alight3) then
         print *, 'from LOSS_ASYM2: freq=alight3'
         goto 200
       else
         jlight3=0
       endif
c
c      radiation modes of effective waveguide
c
       akz1=sqrt(eps_clad1*freq2-akmod*akmod+i*0d0)
       akz2=sqrt(eps_core *freq2-akmod*akmod)
       akz3=sqrt(eps_clad3*freq2-akmod*akmod+i*0d0)
       if (dimag(akz1).gt.0d0) akz1=dconjg(akz1)
       if (dimag(akz3).gt.0d0) akz3=dconjg(akz3)
c
c      Density of states. For asymmetric waveguides: 
c      akz1 for states outgoing to the left  (j=1)
c      akz3 for states outgoing to the right (j=2)
c      DOS is nonzero only above respective light line.
c
       if (jlight1.eq.1) then
         if (dble(akz1).eq.0d0) stop 'error in akz1'
         dos1=eps_clad1/4d0/pi/dble(akz1)
       else if (jlight1.eq.0) then
         dos1=0d0
       else 
         stop 'from LOSS_ASYM2: jlight1 must be 0 or 1'
       endif
       if (jlight3.eq.1) then
         if (dble(akz3).eq.0d0) stop 'error in akz3'
         dos3=eps_clad3/4d0/pi/dble(akz3)
       else if (jlight3.eq.0) then
         dos3=0d0
       else 
         stop 'from LOSS_ASYM2: jlight3 must be 0 or 1'
       endif
c
c      TE modes
c
c      j=1 --> modes which are outgoing from the left
c      j=2 --> modes which are outgoing from the right
c
       aq1=akz1
       aq2=akz2
       aq3=akz3
       t11half=(aq2+aq1)*exp( i*akz2*d/2d0)/2d0/aq2
       t12half=(aq2-aq1)*exp( i*akz2*d/2d0)/2d0/aq2
       t21half=(aq2-aq1)*exp(-i*akz2*d/2d0)/2d0/aq2
       t22half=(aq2+aq1)*exp(-i*akz2*d/2d0)/2d0/aq2
       t11=((aq3+aq2)*(aq2+aq1)*exp( i*akz2*d)
     &     +(aq3-aq2)*(aq2-aq1)*exp(-i*akz2*d))/4d0/aq3/aq2
       t22=((aq3+aq2)*(aq2+aq1)*exp(-i*akz2*d)
     &     +(aq3-aq2)*(aq2-aq1)*exp( i*akz2*d))/4d0/aq3/aq2
       t12=((aq3+aq2)*(aq2-aq1)*exp( i*akz2*d)
     &     +(aq3-aq2)*(aq2+aq1)*exp(-i*akz2*d))/4d0/aq3/aq2
       t21=((aq3-aq2)*(aq2+aq1)*exp( i*akz2*d)
     &     +(aq3+aq2)*(aq2-aq1)*exp(-i*akz2*d))/4d0/aq3/aq2

       det=t11*t22-t12*t21

c
c      outgoing state to the left: the relevant component is xn1(1)
c      and is normalized to 1/sqrt(eps_clad1) to correspond to the DOS
c
       wn1(1)=-t12/t11/sqrt(eps_clad1)
       xn1(1)= 1d0    /sqrt(eps_clad1)
       wn2(1)=t11half*wn1(1)+t12half*xn1(1)
       xn2(1)=t21half*wn1(1)+t22half*xn1(1)
       wn3(1)= 0d0
       xn3(1)= det/t11/sqrt(eps_clad1)

c
c      outgoing state to the right: the relevant component is wn3(2)
c      and is normalized to 1/sqrt(eps_clad3) to correspond to the DOS
c
       wn1(2)= 1d0/t11/sqrt(eps_clad3)
       xn1(2)= 0d0
       wn2(2)=t11half*wn1(2)+t12half*xn1(2)
       xn2(2)=t21half*wn1(2)+t22half*xn1(2)
       wn3(2)= 1d0    /sqrt(eps_clad3)
       xn3(2)= t21/t11/sqrt(eps_clad3)


       if (jlight1.eq.1) write (91,22) 
!     &    wn1(1),xn1(1),wn2(1),xn2(1),wn3(1),xn3(1)
!       write (92,22) wn1(2),xn1(2),wn2(2),xn2(2),wn3(2),xn3(2)
22     format(12(1x,e9.3))

c
c      checks on coefficients: 1=orthogonality, 2+3=normalization
c
       sll=(wn1(1)*dconjg(wn1(1))+xn1(1)*dconjg(xn1(1)))*dble(akz1)
     &    +(wn3(1)*dconjg(wn3(1))+xn3(1)*dconjg(xn3(1)))*dble(akz3)
       srr=(wn1(2)*dconjg(wn1(2))+xn1(2)*dconjg(xn1(2)))*dble(akz1)
     &    +(wn3(2)*dconjg(wn3(2))+xn3(2)*dconjg(xn3(2)))*dble(akz3)
       slr=(wn1(1)*dconjg(wn1(2))+xn1(1)*dconjg(xn1(2)))*dble(akz1)
     &    +(wn3(1)*dconjg(wn3(2))+xn3(1)*dconjg(xn3(2)))*dble(akz3)
       check1=abs(dos1*sll*2d0*pi-1d0)
       check2=abs(dos3*srr*2d0*pi-1d0)
       check3=abs(slr)
       check4=abs(wn3(2)-(t11*wn1(2)+t12*xn1(2)))
       check5=abs(xn3(2)-(t21*wn1(2)+t22*xn1(2)))
       check6=abs(aq3*det-aq1)

       if (jlight1.eq.1.and.jlight3.eq.1) then
         checkmax=max(check1,check2,check3,check4,check5,check6)
       else if (jlight1.eq.1.and.jlight3.eq.0) then
         checkmax=max(check1,check4,check5,check6)
       else if (jlight1.eq.0.and.jlight3.eq.1) then
         checkmax=max(check2,check4,check5,check6)
       else
         checkmax=max(check4,check5,check6)
       endif
       if (checkmax.gt.upper) then
         print *, 'error in LOSS_ASYM2 while calculating TE: checkmax='
     &            , checkmax
       endif
!       if (checkmax.gt.upper) stop 'checkmax too large'
512    format(1x,6(e9.3,',',e9.3,1x))


c
c      Alternative definition of TE modes: symmetric and asymmetric
c      and their linear combinations.
c      The sym and asym modes are those which are used 
c      in subroutine loss_sym: Their linear combinations
c      coincide exactly with the coefficients defined above.
c      Thus in the case of a symmetric waveguide, the two approaches 
c      and subroutines give exactly the same results.
c
!       xn1(1)=1d0/sqrt(2d0*eps_clad1)
!       xn1(2)=1d0/sqrt(2d0*eps_clad1)
!       wn1(1)=(1d0-t12)/t11*xn1(1) 
!       wn1(2)=-(1d0+t12)/t11*xn1(2)
!       do j=1,2
!         wn2(j)=t11half*wn1(j)+t12half*xn1(j)
!         xn2(j)=t21half*wn1(j)+t22half*xn1(j)
!         wn3(j)=t11    *wn1(j)+t12    *xn1(j)
!         xn3(j)=t21    *wn1(j)+t22    *xn1(j)
!       enddo

!       wn1c(1)=(wn1(1)+wn1(2))/sqrt(2d0)
!       xn1c(1)=(xn1(1)+xn1(2))/sqrt(2d0)
!       wn2c(1)=(wn2(1)+wn2(2))/sqrt(2d0)
!       xn2c(1)=(xn2(1)+xn2(2))/sqrt(2d0)
!       wn3c(1)=(wn3(1)+wn3(2))/sqrt(2d0)
!       xn3c(1)=(xn3(1)+xn3(2))/sqrt(2d0)
!       wn1c(2)=(wn1(1)-wn1(2))/sqrt(2d0)
!       xn1c(2)=(xn1(1)-xn1(2))/sqrt(2d0)
!       wn2c(2)=(wn2(1)-wn2(2))/sqrt(2d0)
!       xn2c(2)=(xn2(1)-xn2(2))/sqrt(2d0)
!       wn3c(2)=(wn3(1)-wn3(2))/sqrt(2d0)
!       xn3c(2)=(xn3(1)-xn3(2))/sqrt(2d0)

!       write (91,22) wn1c(1),xn1c(1),wn2c(1),xn2c(1),wn3c(1),xn3c(1)
!       write (92,22) wn1c(2),xn1c(2),wn2c(2),xn2c(2),wn3c(2),xn3c(2)
!       write (91,*)
!       write (92,*)

c
c      TM modes
c
       aq1=akz1/eps_clad1
       aq2=akz2/eps_core
       aq3=akz3/eps_clad3
       t11half=(aq2+aq1)*exp( i*akz2*d/2d0)/2d0/aq2
       t12half=(aq2-aq1)*exp( i*akz2*d/2d0)/2d0/aq2
       t21half=(aq2-aq1)*exp(-i*akz2*d/2d0)/2d0/aq2
       t22half=(aq2+aq1)*exp(-i*akz2*d/2d0)/2d0/aq2
       t11=((aq3+aq2)*(aq2+aq1)*exp( i*akz2*d)
     &     +(aq3-aq2)*(aq2-aq1)*exp(-i*akz2*d))/4d0/aq3/aq2
       t22=((aq3+aq2)*(aq2+aq1)*exp(-i*akz2*d)
     &     +(aq3-aq2)*(aq2-aq1)*exp( i*akz2*d))/4d0/aq3/aq2
       t12=((aq3+aq2)*(aq2-aq1)*exp( i*akz2*d)
     &     +(aq3-aq2)*(aq2+aq1)*exp(-i*akz2*d))/4d0/aq3/aq2
       t21=((aq3-aq2)*(aq2+aq1)*exp( i*akz2*d)
     &     +(aq3+aq2)*(aq2-aq1)*exp(-i*akz2*d))/4d0/aq3/aq2

       det=t11*t22-t12*t21

c
c      outgoing state to the left: the relevant component is zn1(1)
c      and is normalized to unity to correspond to the DOS
c
       yn1(1)=-t12/t11
       zn1(1)= 1d0
       yn2(1)=t11half*yn1(1)+t12half*zn1(1)
       zn2(1)=t21half*yn1(1)+t22half*zn1(1)
       yn3(1)= 0d0
       zn3(1)= det/t11

c
c      outgoing state to the right: the relevant component is yn3(2)
c      and is normalized to unity to correspond to the DOS
c
       yn1(2)= 1d0/t11
       zn1(2)= 0d0
       yn2(2)=t11half*yn1(2)+t12half*zn1(2)
       zn2(2)=t21half*yn1(2)+t22half*zn1(2)
       yn3(2)= 1d0
       zn3(2)= t21/t11


!       print *, 'from LOSS_ASYM2: starting check of orthogonality TM'
!       print *, akx,aky,freq/2d0/pi,jband


c
c      checks on coefficients: 1=orthogonality, 2+3=normalization
c
       sll=(yn1(1)*dconjg(yn1(1))+zn1(1)*dconjg(zn1(1)))*dble(aq1)
     &    +(yn3(1)*dconjg(yn3(1))+zn3(1)*dconjg(zn3(1)))*dble(aq3)
       srr=(yn1(2)*dconjg(yn1(2))+zn1(2)*dconjg(zn1(2)))*dble(aq1)
     &    +(yn3(2)*dconjg(yn3(2))+zn3(2)*dconjg(zn3(2)))*dble(aq3)
       slr=(yn1(1)*dconjg(yn1(2))+zn1(1)*dconjg(zn1(2)))*dble(aq1)
     &    +(yn3(1)*dconjg(yn3(2))+zn3(1)*dconjg(zn3(2)))*dble(aq3)
       check1=abs(dos1*sll*2d0*pi-1d0)
       check2=abs(dos3*srr*2d0*pi-1d0)
       check3=abs(slr)
       check4=abs(yn3(2)-(t11*yn1(2)+t12*zn1(2)))
       check5=abs(zn3(2)-(t21*yn1(2)+t22*zn1(2)))
       check6=abs(aq3*det-aq1)
       if (jlight1.eq.1.and.jlight3.eq.1) then
         checkmax=max(check1,check2,check3,check4,check5,check6)
       else if (jlight1.eq.1.and.jlight3.eq.0) then
         checkmax=max(check1,check4,check5,check6)
       else if (jlight1.eq.0.and.jlight3.eq.1) then
         checkmax=max(check2,check4,check5,check6)
       else 
         checkmax=max(check4,check5,check6)
       endif
       if (checkmax.gt.upper) then
         print *, 'error in LOSS_ASYM2 while calculating TM: checkmax'
     &           , checkmax
       endif
!       if (checkmax.gt.upper) stop 'checkmax too large'

       pol2(1)= agx2/akmod
       pol2(2)= agy2/akmod
       qol2(1)=-agy2/akmod
       qol2(2)= agx2/akmod

c
c      matrix element with guided mode: sum over basis states
c      for j=1,2 --> outgoing states from left and right
c 
       do 50 j=1,2
        elemte(j)=dcmplx(0d0,0d0)
        elemtm(j)=dcmplx(0d0,0d0)
        if (j.eq.1.and.alight1.eq.0) goto 50 ! below light line in medium 1
        if (j.eq.2.and.alight3.eq.0) goto 50 ! below light line in medium 3

        do 100 j1=1,npw
        do 100 jal1=1,nalpha_store(j1)
         jham1=jham_store(j1,jal1)
         jmode1=jmode_store(j1,jal1)
         omega1=om_store(j1,jal1)
         agx1=akx+gxvec(j1)
         agy1=aky+gyvec(j1)
         agmod=sqrt(agx1*agx1+agy1*agy1)
         q=q_store(j1,jal1) 
         chi1=chi1_store(j1,jal1)
         chi3=chi3_store(j1,jal1)
         pol1(1)= agx1/agmod
         pol1(2)= agy1/agmod
         qol1(1)=-agy1/agmod
         qol1(2)= agx1/agmod

         aiminus=d*spher0((q-akz2)*d/2d0)
         aiplus =d*spher0((q+akz2)*d/2d0)

c
c        checks denominators are nonzero
c
         if (chi1+i*akz1.eq.0) stop 'chi1+i*akz1=0'
         if (chi1-i*akz1.eq.0) stop 'chi1-i*akz1=0'
         if (chi3+i*akz3.eq.0) stop 'chi3+i*akz3=0'
         if (chi3-i*akz3.eq.0) stop 'chi3-i*akz3=0'


         if (jmode1.eq.0) then ! TE polarized basis state

          ainttete(j)=omega1*omega1*freq*scal(qol1,qol2,2) * (
     &    eps_clad1*eps_clad1*ft1inv(j1,j2)*dconjg(bnorm1(j1,jal1))
     &    *(wn1(j)/(chi1+i*akz1)+xn1(j)/(chi1-i*akz1))
     &   +eps_clad3*eps_clad3*ft3inv(j1,j2)*dconjg(anorm3(j1,jal1))
     &    *(wn3(j)/(chi3-i*akz3)+xn3(j)/(chi3+i*akz3))
     &   +eps_core*eps_core*ft2inv(j1,j2)*(
     &    (dconjg(anorm2(j1,jal1))*wn2(j)
     &    +dconjg(bnorm2(j1,jal1))*xn2(j))*aiminus
     &   +(dconjg(anorm2(j1,jal1))*xn2(j)
     &    +dconjg(bnorm2(j1,jal1))*wn2(j))*aiplus ) 
     &     )

          ainttetm(j)=i*omega1*omega1*scal(qol1,pol2,2)* (
     &    eps_clad1*ft1inv(j1,j2)*dconjg(bnorm1(j1,jal1))
     &    *akz1*(-yn1(j)/(chi1+i*akz1)+zn1(j)/(chi1-i*akz1))
     &   +eps_clad3*ft3inv(j1,j2)*dconjg(anorm3(j1,jal1))
     &    *akz3*(-yn3(j)/(chi3-i*akz3)+zn3(j)/(chi3+i*akz3))
     &   +eps_core*ft2inv(j1,j2)*akz2* (
     &    (-dconjg(anorm2(j1,jal1))*yn2(j)
     &     +dconjg(bnorm2(j1,jal1))*zn2(j))*aiminus
     &   +( dconjg(anorm2(j1,jal1))*zn2(j)
     &     -dconjg(bnorm2(j1,jal1))*yn2(j))*aiplus ) 
     &      )

          couplte(j)=ainttete(j)
          coupltm(j)=ainttetm(j)

         else if (jmode1.eq.1)  then ! TM polarized basis state

          ainttmte(j)=freq*scal(pol1,qol2,2)* (
     &    -eps_clad1*ft1inv(j1,j2)*chi1*dconjg(dnorm1(j1,jal1))
     &    *(wn1(j)/(chi1+i*akz1)+xn1(j)/(chi1-i*akz1))
     &    +eps_clad3*ft3inv(j1,j2)*chi3*dconjg(cnorm3(j1,jal1))
     &    *(wn3(j)/(chi3-i*akz3)+xn3(j)/(chi3+i*akz3))
     &    -i*eps_core*ft2inv(j1,j2)*q* (
     &    (-dconjg(cnorm2(j1,jal1))*wn2(j)
     &     +dconjg(dnorm2(j1,jal1))*xn2(j))*aiminus
     &    +(dconjg(dnorm2(j1,jal1))*wn2(j)
     &     -dconjg(cnorm2(j1,jal1))*xn2(j))*aiplus ) 
     &      )


          ainttmtm(j)=
     &    +ft1inv(j1,j2)*dconjg(dnorm1(j1,jal1)) * (
     &      (i*chi1*akz1*scal(pol1,pol2,2)+agmod*akmod)
     &      *yn1(j)/(chi1+i*akz1)
     &     +(-i*chi1*akz1*scal(pol1,pol2,2)+agmod*akmod)
     &      *zn1(j)/(chi1-i*akz1)                  )
     &    +ft3inv(j1,j2)*dconjg(cnorm3(j1,jal1)) * (
     &      (-i*chi3*akz3*scal(pol1,pol2,2)+agmod*akmod)
     &      *yn3(j)/(chi3-i*akz3)
     &     +( i*chi3*akz3*scal(pol1,pol2,2)+agmod*akmod)
     &      *zn3(j)/(chi3+i*akz3)                  )
     &    +ft2inv(j1,j2)*(
     &     (dconjg(cnorm2(j1,jal1))*yn2(j)
     &     +dconjg(dnorm2(j1,jal1))*zn2(j))
     &    *( q*akz2*scal(pol1,pol2,2)+agmod*akmod)*aiminus
     &    +(dconjg(cnorm2(j1,jal1))*zn2(j)
     &     +dconjg(dnorm2(j1,jal1))*yn2(j))
     &    *(-q*akz2*scal(pol1,pol2,2)+agmod*akmod)*aiplus)

          couplte(j)=ainttmte(j)
          coupltm(j)=ainttmtm(j)

         endif


c
c        adds contribution from basis state (j1,jal1)-->jham1
c

         elemte(j)=elemte(j)+couplte(j)*eigenv(jham1,jband)
         elemtm(j)=elemtm(j)+coupltm(j)*eigenv(jham1,jband)

100     continue      ! ends loop over basis state indices j1,jal1
50     continue       ! ends loop over left- right- index j=1,2 

c 
c      imaginary part of squared frequency
c
       add=pi*(
     &  (elemte(1)*dconjg(elemte(1))+elemtm(1)*dconjg(elemtm(1)))*dos1
     & +(elemte(2)*dconjg(elemte(2))+elemtm(2)*dconjg(elemtm(2)))*dos3
     &        )
       freq2im=freq2im+add
200   continue           ! ends loop over rec. lattice vector or BZ

c
c     imaginary part of frequency --> square root
c
      freqim=dimag(sqrt(freq2+i*freq2im))
c
c     ends calculation of diffraction losses for given wavevector
c
      return
      end
c
      subroutine initialize (a,nmax,n)
      implicit none
      integer nmax,n,j1,j2
      real*8 a(nmax,nmax)
      do 1 j1=1,n
      do 1 j2=1,n
        a(j1,j2)=0d0
1     continue
      return
      end
c
      subroutine checksym (a,max,n)
c
c     checks matrix A is symmetric
c
      implicit none
      integer max,n,j1,j2
      real*8 a(max,max),upper,diff
      parameter (upper=1d-11)
      if (n.gt.max) stop ' from CHECKSYM: n>max'
      do j1=1,n
      do j2=j1,n
        diff=a(j1,j2)-a(j2,j1)
        if (abs(diff).gt.upper) then
          write (6,*) ' from CHECKSYM: matrix A not symmetric'
          write (6,*) ' j1,j2,diff=',j1,j2,diff
          stop
        endif
      enddo
      enddo
      return
      end
c
      subroutine checkherm (a,max,n)
c
c     checks matrix A is hermitian
c
      implicit none
      integer max,n,j1,j2
      real*8 upper,diff
      complex*16 a(max,max)
      parameter (upper=1d-11)
      if (n.gt.max) stop ' from CHECKHERM: n>max'
      do j1=1,n
      do j2=j1,n
        diff=a(j1,j2)-dconjg(a(j2,j1))
        if (abs(diff).gt.upper) then
          write (6,*) ' from CHECKHERM: matrix A not hermitian'
          write (6,*) ' j1,j2,diff=',j1,j2,diff
          stop
        endif
      enddo
      enddo
      return
      end
c
      subroutine repulisti (a,max,n)
c
c     sets to zero elements smaller than UPPER
c
      implicit none
      integer max,n,j1,j2
      real*8 upper,a(max,max)
      parameter (upper=1d-12)
      do j1=1,n
      do j2=1,n
        if (abs(a(j1,j2)).lt.upper) a(j1,j2)=0d0
      enddo
      enddo
      return
      end
c
      double precision function cosint (q1,q2,d)
      implicit none
      real*8 q1,q2,d,spher0
      cosint=d/2d0*(spher0((q1-q2)*d/2d0)+spher0((q1+q2)*d/2d0))
      return
      end
c
      double precision function sinint (q1,q2,d)
      implicit none
      real*8 q1,q2,d,spher0
      sinint=d/2d0*(spher0((q1-q2)*d/2d0)-spher0((q1+q2)*d/2d0))
      return
      end
c
      subroutine block (ablock,maxblock,nblock,a,maxa,jbl1,jbl2)
c
c     writes block matrix ABLOCK in matrix A, 
c     starting at positions (JBL1,JBL2)
c
      implicit none
      integer maxblock,nblock,maxa,jbl1,jbl2,j1,j2
      real*8 ablock(maxblock,maxblock),a(maxa,maxa)
      if (nblock.gt.maxblock) stop ' from BLOCK: nblock>maxblock'
      if (jbl1.lt.0) stop ' from BLOCK: jbl1<0'
      if (jbl2.lt.0) stop ' from BLOCK: jbl2<0'
      if (jbl1+nblock.gt.maxa) stop ' from BLOCK: jbl1+nblock>maxa'
      if (jbl2+nblock.gt.maxa) stop ' from BLOCK: jbl2+nblock>maxa'
      do j1=1,nblock
      do j2=1,nblock
        a(j1+jbl1,j2+jbl2)=ablock(j1,j2)
      enddo
      enddo
      return
      end
c
      double precision function spher0 (x)
c
c     spherical Bessel function of order 0: sin(x)/x
c
      implicit none
      real*8 x
      if (x.eq.0) then
        spher0=1d0
      else 
        spher0=sin(x)/x
      endif
      return
      end
c
      subroutine checkuc (x,y,a1,a2,juc)
c
c     Checks whether point (x,y) belongs to unit cell of 2D lattice
c     defined by primitive vectors a1,a2.
c     Strategy: considers several nonzero lattice points (x1,y1)
c     and checks whether (x,y) is closer to origin than to (x1,y1).
c    
c     Output: juc=0 if point does not belong to unit cell
c            juc=1 if point does belong to unit cell.
c
      implicit none
      integer juc,n1,n2,nmax
      real*8 x,y,x1,y1,a1(2),a2(2),amod,amod1
      nmax=3
      juc=1
      amod=x*x+y*y
      do n1=-nmax,nmax
      do n2=-nmax,nmax
        if (n1.ne.0.or.n2.ne.0) then
          x1=n1*a1(1)+n2*a2(1)
          y1=n1*a1(2)+n2*a2(2)
          amod1=(x-x1)*(x-x1)+(y-y1)*(y-y1)
          if (amod.gt.amod1) then
            juc=0
            return
          endif
        endif
      enddo
      enddo
      return
      end
c
      subroutine grating_points (g,x,y,am)
c
c     main symmetry points of BZ of lattice corresponding to 1D grating, 
c     i.e., periodic along x and homogeneous along y
c
      implicit none
      real*8 g(2),x(2),y(2),am(2),aret,pi
      parameter (aret=1d0)
      pi=4d0*atan(1d0)

      g(1)=0d0
      g(2)=0d0

      x(1)=pi/aret
      x(2)=0d0

      y(1)=0d0
      y(2)=pi/aret

      am(1)=pi/aret
      am(2)=pi/aret

      return
      end
C
      subroutine square_points (g,x,am)
c
c     main symmetry points of BZ of square lattice
c
      implicit none
      real*8 g(2),x(2),am(2),aret,pi
      parameter (aret=1d0)
      pi=4d0*atan(1d0)

      g(1)=0d0
      g(2)=0d0

      x(1)=pi/aret
      x(2)=0d0

      am(1)=pi/aret
      am(2)=pi/aret

      return
      end
c
      subroutine triang_points (g,am,ak)
c
c     main symmetry points of BZ of triangular lattice
c
      implicit none
      real*8 g(2),ak(2),am(2),aret,pi
      parameter (aret=1d0)
      pi=4d0*atan(1d0)

      g(1)=0d0
      g(2)=0d0

      am(1)=0d0
      am(2)=2d0*pi/sqrt(3d0)/aret

c
c     the two definitions below are equivalent, but if you
c     change from one to the other, subroutine "decouwga" 
c     has to be modified 
c
      ak(1)=2d0*pi/3d0/aret
      ak(2)=2d0*pi/sqrt(3d0)/aret
!      ak(1)=4d0*pi/3d0/aret
!      ak(2)=0d0

      return
      end
c
      subroutine w1_points (g,x1,x2,am)
c
c     main symmetry points of BZ of square lattice
c
      implicit none
      real*8 g(2),x1(2),x2(2),am(2),aret,pi
      real*8 alength1,alength2,alength3,dummy
      parameter (aret=1d0)
      common/length/alength1,alength2,alength3,dummy
      pi=4d0*atan(1d0)

      g(1)=0d0
      g(2)=0d0

      x1(1)=pi/aret
      x1(2)=0d0

      x2(1)=0d0
      x2(2)=pi/aret/(alength2+alength3)/sqrt(3d0)

      am(1)=pi/aret
      am(2)=pi/aret/(alength2+alength3)/sqrt(3d0)

      return
      end
c
      subroutine kline (p,q,akx,aky,ax,maxk,nk,jstart)
c
c     defines line in BZ from point p to q
c
      implicit none
      integer maxk,nk,jstart,jk
      real*8 p(2),q(2),akx(maxk),aky(maxk),ax(maxk),delta,dist
      
      if (jstart+nk.gt.maxk) stop ' from kline: jstart+nk>maxk'
      if (nk.ne.0) delta=1d0/nk
      dist=sqrt( (q(1)-p(1))*(q(1)-p(1)) + (q(2)-p(2))*(q(2)-p(2)) )
      do jk=0,nk
        akx(jstart+jk)=p(1)+(q(1)-p(1))*jk*delta
        aky(jstart+jk)=p(2)+(q(2)-p(2))*jk*delta
        ax (jstart+jk)=ax(jstart)+dist*jk*delta 
      enddo
      return
      end
c
      FUNCTION BESSJ1(X)
c
c     from Numerical Recipes
c
      implicit real*8 (a-h,o-z)
      REAL*8 Y,P1,P2,P3,P4,P5,Q1,Q2,Q3,Q4,Q5,R1,R2,R3,R4,R5,R6,
     *    S1,S2,S3,S4,S5,S6
      DATA R1,R2,R3,R4,R5,R6/72362614232.D0,-7895059235.D0,242396853.1D0
     *,
     *    -2972611.439D0,15704.48260D0,-30.16036606D0/,
     *    S1,S2,S3,S4,S5,S6/144725228442.D0,2300535178.D0,
     *    18583304.74D0,99447.43394D0,376.9991397D0,1.D0/
      DATA P1,P2,P3,P4,P5/1.D0,.183105D-2,-.3516396496D-4,.2457520174D-5
     *,
     *    -.240337019D-6/, Q1,Q2,Q3,Q4,Q5/.04687499995D0,-.2002690873D-3
     *,
     *    .8449199096D-5,-.88228987D-6,.105787412D-6/
      IF(ABS(X).LT.8.)THEN
        Y=X**2
        BESSJ1=X*(R1+Y*(R2+Y*(R3+Y*(R4+Y*(R5+Y*R6)))))
     *      /(S1+Y*(S2+Y*(S3+Y*(S4+Y*(S5+Y*S6)))))
      ELSE
        AX=ABS(X)
        Z=8.d0/AX
        Y=Z**2
        XX=AX-2.356194491
        BESSJ1=SQRT(.636619772/AX)*(COS(XX)*(P1+Y*(P2+Y*(P3+Y*(P4+Y
     *      *P5))))-Z*SIN(XX)*(Q1+Y*(Q2+Y*(Q3+Y*(Q4+Y*Q5)))))
     *      *SIGN(1.d0,X)
      ENDIF
      RETURN
      END
c
      SUBROUTINE SORT5(N,RA,RB,RC,ND,NE,WKSP,IWKSP,nwksp)
c
c     Sorts array RA in increasing order and reorders also
c     arrays RB,RC,ND,NE according to the same indexing.
c     Modified from Numerical Recipes
c
      implicit real*8 (a-h,o-z)
      DIMENSION RA(N),RB(N),RC(N),ND(N),NE(N),WKSP(N),IWKSP(N),nwksp(n)
      CALL INDEXX(N,RA,IWKSP)
      DO 11 J=1,N
        WKSP(J)=RA(J)
11    CONTINUE
      DO 12 J=1,N
        RA(J)=WKSP(IWKSP(J))
12    CONTINUE
      DO 13 J=1,N
        WKSP(J)=RB(J)
13    CONTINUE
      DO 14 J=1,N
        RB(J)=WKSP(IWKSP(J))
14    CONTINUE
      DO 15 J=1,N
        WKSP(J)=RC(J)
15    CONTINUE
      DO 16 J=1,N
        RC(J)=WKSP(IWKSP(J))
16    CONTINUE
      DO 17 J=1,N
        NWKSP(J)=ND(j)
17    CONTINUE
      DO 18 J=1,N
        ND(j)=NWKSP(IWKSP(J))
18    CONTINUE
      DO 19 J=1,N
        NWKSP(J)=NE(J)
19    CONTINUE
      DO 20 J=1,N
        NE(J)=NWKSP(IWKSP(J))
20    CONTINUE
      RETURN
      END
c
c     *************************************************************
c
      SUBROUTINE INDEXX(N,ARRIN,INDX)
c
c     From Numerical Recipes
c
      implicit real*8 (a-h,o-z)
      DIMENSION ARRIN(N),INDX(N)
      DO 11 J=1,N
        INDX(J)=J
11    CONTINUE
      L=N/2+1
      IR=N
10    CONTINUE
        IF(L.GT.1)THEN
          L=L-1
          INDXT=INDX(L)
          Q=ARRIN(INDXT)
        ELSE
          INDXT=INDX(IR)
          Q=ARRIN(INDXT)
          INDX(IR)=INDX(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            INDX(1)=INDXT
            RETURN
          ENDIF
        ENDIF
        I=L
        J=L+L
20      IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(ARRIN(INDX(J)).LT.ARRIN(INDX(J+1)))J=J+1
          ENDIF
          IF(Q.LT.ARRIN(INDX(J)))THEN
            INDX(I)=INDX(J)
            I=J
            J=J+J
          ELSE
            J=IR+1
          ENDIF
        GO TO 20
        ENDIF
        INDX(I)=INDXT
      GO TO 10
      END
c
      subroutine savemt (a,b,nmax,n)
      implicit none
      integer nmax,n,j1,j2
      real*8 a(nmax,nmax),b(nmax,nmax)
      do j1=1,n
      do j2=1,n
        b(j1,j2)=a(j1,j2)
      enddo
      enddo
      return
      end
c
      double precision function delta_kronecker (j1,j2)
c
c     Kronecker's delta
c
      implicit none
      integer j1,j2
      if (j1.eq.j2) then
        delta_kronecker=1d0
      else
        delta_kronecker=0d0
      endif
      return
      end
c
      subroutine checkinv (a,b,nmax,n)
      implicit none
      integer nmax,n,j1,j2,j3
      real*8 a(nmax,nmax),b(nmax,nmax),sum,upper
      parameter (upper=1d-12)
      do j1=1,n
      do j2=1,n
        sum=0d0
        do j3=1,n
          sum=sum+a(j1,j3)*b(j3,j2)
        enddo
        if (j1.eq.j2) then
          if (abs(sum-1d0).gt.upper) stop ' from checkinv: error'
        else
          if (abs(sum).gt.upper) stop ' from checkinv: error'
        endif
      enddo
      enddo
      return
      end
c
      double precision function scal(a,b,n)
c
c     scalar product of arrays
c
      implicit none
      integer n,j
      real*8 a(n),b(n)
      scal=0d0
      do j=1,n
        scal=scal+a(j)*b(j)
      enddo
      return
      end
c
      SUBROUTINE WRMAT (A,NMAX,N,FACPRO,FACSUM)
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 A(NMAX,NMAX)
      WRITE (2,*)
C
C     NUMBER OF TIMES MUST WRITE BLOCKS OF COLUMNS
C
      NWRITE=1+(N-1)/10
      DO 30 IWRITE=1,NWRITE
C
C       N1,..,N2=NUMBER OF COLUMNS MUST BE WRITTEN IN THE PRESENT BLOCK
C
        N1=1+10*(IWRITE-1)
        N2=MIN(10*IWRITE,N)
        IF (NWRITE.GT.1) WRITE (2,5) N1,N2
5       FORMAT(' COLUMNS ',I2,' - ',I2,/)
        DO 10 I=1,N
10        WRITE (2,20) ((A(I,J)+FACSUM)*FACPRO,J=N1,N2)
20        FORMAT(1X,10(E10.4,2X))
        WRITE (2,*)
30    CONTINUE
      WRITE (2,*)
      RETURN
      END
C
      SUBROUTINE WRVEC (A,NMAX,N,FACPRO,FACSUM)
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 A(NMAX)
      WRITE (2,*)
      NWRITE=1+(N-1)/10
      DO 30 IWRITE=1,NWRITE
        N1=1+10*(IWRITE-1)
        N2=MIN(10*IWRITE,N)
        WRITE (2,20) ((A(J)+FACSUM)*FACPRO,J=N1,N2)
20      FORMAT(1X,10(E10.4,2X))
        WRITE (2,*)
30    CONTINUE
      RETURN
      END
C
      SUBROUTINE WRMAT2 (A,NMAX,NBLOCK,NDIM,JBLOCK,FACPRO,FACSUM)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (N2=40)
      REAL*8 A(NMAX,NMAX),BLOCK(N2,N2)
      IF (NDIM.GT.N2) THEN
        WRITE (6,2) N2
2       FORMAT(' FROM MODULE WRMAT2: BLOCK DIMENSION EXCEEDS NMAX=',I2)
        WRITE (6,*) ' NO MATRIX WRITTEN'
        RETURN
      ENDIF
C
C     DEFINES BLOCK TO BE WRITTEN
C
      DO 10 I=1,NDIM
       DO 10 J=1,NDIM
10      BLOCK(I,J)=A((JBLOCK-1)*NDIM+I,(JBLOCK-1)*NDIM+J)
      CALL WRMAT (BLOCK,N2,NDIM,FACPRO,FACSUM)
      RETURN
      END
C
      SUBROUTINE WRVEC2 (A,NMAX,NBLOCK,NDIM,JBLOCK,FACPRO,FACSUM)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (N2=40)
      REAL*8 A(NMAX),BLOCK(N2)
      IF (NDIM.GT.N2) THEN
        WRITE (6,2) N2
2       FORMAT(' FROM MODULE WRVEC2: BLOCK DIMENSION EXCEEDS NMAX=',I2)
        WRITE (6,*) ' NO VECTOR WRITTEN'
        RETURN
      ENDIF
C
C     DEFINES BLOCK TO BE WRITTEN
C
      DO 10 I=1,NDIM
10      BLOCK(I)=A((JBLOCK-1)*NDIM+I)
      CALL WRVEC (BLOCK,N2,NDIM,FACPRO,FACSUM)
      RETURN
      END
c
      SUBROUTINE SETACC (NZERI,NFOUND,XMIN,XMAX,IMAX,I,F,A,B)
C
C     ***************************************************************
C
C        L. C. ANDREANI AND A. PASQUARELLO, PISA, 1986
C
c        Bracketing of zeros: finds extrema of intervals
c        containing a specified number of zeros of function F(X).
C
C        NZERI       (ENTRY) = Number of required zeros (unmodified)
C        NFOUND      (EXIT)  = Number of zeros found
C        XMIN , XMAX (ENTRY) = extrema of bracketing region
C                              (xmin is modified if F(xmin)=0)
C        IMAX        (ENTRY) = Maximum number of steps (unmodified)
C        I           (EXIT)  = Actual number of steps
C        F=F(X)      (ENTRY) = External function (unmodified)
C        A , B       (EXIT)  = Arrays with extrema of intervals
c                              bracketing a zero
C
C     ****************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REAL*8 A(NZERI),B(NZERI)
C
C     ****************************************************************
C
C             DELTA = PASSO DELLA RICERCA
C             FRACT = PARAMETRO DI AGGIUSTAMENTO
C             N = CONTATORE ZERI INDIVIDUATI
C
C     ****************************************************************
C
      PARAMETER (FRACT=.001)
      DELTA=(XMAX-XMIN)/IMAX
      NFOUND=0
C
C     CALCOLA IL PRIMO SEGNO E, SE F=0, ARRETRA
C
      NCONTR=0
10    IF (NCONTR.EQ.4) THEN
         WRITE (6,*) 'FROM MODULE SETACC: FUNCTION IDENTICALLY ZERO'
         STOP
      END IF
      F1=F(XMIN)
      S1=SIGN(1D0,F1)
      IF (F1.EQ.0.) THEN
         XMIN=XMIN-DELTA*FRACT
         NCONTR=NCONTR+1
         GOTO 10
      END IF
C
C     ***************************************************************
C     *  LOOP CHE SETACCIA L' INTERVALLO ASSEGNATO
C     *  CALCOLA I SEGNI AI DUE ESTREMI, E LI CONFRONTA:
C     *  SE SONO UGUALI PROSEGUE , SE SONO DIVERSI ...
C     *  ... HA TROVATO UNO ZERO, PERCIO' INCREMENTA NFOUND
C     ***************************************************************
C
      DO 100 I=1,IMAX
         X=XMIN+I*DELTA
         NCONTR=0
20       IF (NCONTR.EQ.4) THEN
            WRITE (6,*) 'FROM MODULE SETACC: FUNCTION IDENTICALLY ZERO'
            STOP
         END IF
         F2=F(X)
         S2=SIGN(1D0,F2)
         IF (F2.EQ.0.D0) THEN
            NCONTR=NCONTR+1
            X=X+DELTA*FRACT
            GOTO 20
         ELSE IF (S2.NE.S1) THEN
            NFOUND=NFOUND+1
            A(NFOUND)=X-DELTA
            B(NFOUND)=X
            IF (NFOUND.EQ.NZERI) RETURN
            S1=S2
         END IF
100   CONTINUE
      RETURN
      END
C
      SUBROUTINE ZERO (X1,X2,X0,ERROR,DIFF,IMAX,I,F,FLAG)
C
C     ***************************************************************
C
C             L. C. ANDREANI AND A. PASQUARELLO, PISA, 1986
C
C
c     Finds the zero of a function F(x) in a specified interval
c     by straight bisection. To be used after subroutine "SETACC".
C
C     Arguments:
C
C     X1,X2 (ENTRY) = Extrema of interval (unmodified)
C     X0    (EXIT)  = Zero of function
C     ERROR (ENTRY) = Error parameter (unmodified)
C     DIFF  (EXIT)  = actual error; abs(x0-x0true)<diff
C     IMAX  (ENTRY) = Maximum number of iterations (unmodified)
C     I     (EXIT)  = Actual number of iterations
C     F     (ENTRY) = Name of external function F(x) (unmodified)
C     FLAG  (EXIT)  = set equal to 1 if the function has equal sign
C                     at extrema, otherwise set to 0...
C
C     Other variables:
C
C     F0,F1,F2 = Function values
C     S0,S1,S2 = Function signs
C
C     ****************************************************************
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      I=0
      FLAG=0
      F1=F(X1)
      F2=F(X2)
      S1=SIGN(1D0,F1)
      S2=SIGN(1D0,F2)
      IF (F1.EQ.0) THEN
         X0=X1
         DIFF=0
         RETURN
      ELSE IF (F2.EQ.0.) THEN
         X0=X2
         DIFF=0
         RETURN
      ELSE IF (S1.EQ.S2) THEN
         FLAG=1
         RETURN
      END IF
100   X0=(X2+X1)/2
      F0=F(X0)
      S0=SIGN(1D0,F0)
      I=I+1
      IF (F0.EQ.0.) THEN
         DIFF=0
         RETURN
      ELSE IF (S0.EQ.S1) THEN
         X1=X0
         F1=F0
      ELSE
         X2=X0
         F2=F0
      END IF
      IF ((ABS(X2-X1).LT.ERROR).OR.(I.EQ.IMAX)) THEN
         X0=(X2+X1)/2
         DIFF=ABS((X2-X1)/2)
         IF (DIFF.GE.ERROR) THEN
            WRITE (6,*) 'FROM MODULE ZERO: MAXIMUM STEPS TAKEN'
         END IF
         RETURN
      END IF
      GOTO 100
      END

